/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Admin - OrganizationalUnitDisseminationPolicyService.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.business;

import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;
import org.springframework.stereotype.Service;

import ch.unige.solidify.rest.JoinResourceContainer;
import ch.unige.solidify.service.JoinResource2TiersService;
import ch.unige.solidify.specification.Join2TiersSpecification;

import ch.dlcm.controller.AdminController;
import ch.dlcm.model.display.OrganizationalUnitDisseminationPolicyDTO;
import ch.dlcm.model.policies.DisseminationPolicy;
import ch.dlcm.model.policies.OrganizationalUnitDisseminationPolicy;
import ch.dlcm.model.settings.OrganizationalUnit;
import ch.dlcm.specification.OrganizationalUnitDisseminationPolicySpecification;

@Service
@ConditionalOnBean(AdminController.class)
public class OrganizationalUnitDisseminationPolicyService
        extends JoinResource2TiersService<OrganizationalUnit, DisseminationPolicy, OrganizationalUnitDisseminationPolicy> {

  @Override
  public Join2TiersSpecification<OrganizationalUnitDisseminationPolicy> getJoinSpecification(
          OrganizationalUnitDisseminationPolicy orgUnitDisseminationPolicy) {
    return new OrganizationalUnitDisseminationPolicySpecification(orgUnitDisseminationPolicy);
  }

  @Override
  public DisseminationPolicy getChildResource(OrganizationalUnitDisseminationPolicy orgUnitDisseminationPolicy) {
    return orgUnitDisseminationPolicy.getDisseminationPolicy();
  }

  @Override
  public String getChildPathFromJoinResource() {
    return OrganizationalUnitDisseminationPolicy.PATH_TO_DISSEMINATION_POLICY;
  }

  @Override
  public JoinResourceContainer<OrganizationalUnitDisseminationPolicy> getChildDTO(
          OrganizationalUnitDisseminationPolicy orgUnitDisseminationPolicy) {
    return new OrganizationalUnitDisseminationPolicyDTO(orgUnitDisseminationPolicy);
  }

  @Override
  public OrganizationalUnitDisseminationPolicy getEmptyJoinResourceObject() {
    return new OrganizationalUnitDisseminationPolicy();
  }

  @Override
  public OrganizationalUnit getEmptyParentResourceObject() {
    return new OrganizationalUnit();
  }

  @Override
  public DisseminationPolicy getEmptyChildResourceObject() {
    return new DisseminationPolicy();
  }

  @Override
  public void setParentResource(OrganizationalUnitDisseminationPolicy orgUnitDisseminationPolicy, OrganizationalUnit orgUnit) {
    orgUnitDisseminationPolicy.setOrganizationalUnit(orgUnit);
  }

  @Override
  public void setChildResource(OrganizationalUnitDisseminationPolicy orgUnitDisseminationPolicy, DisseminationPolicy policy) {
    orgUnitDisseminationPolicy.setDisseminationPolicy(policy);
  }

}
