/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Admin - OrganizationalUnitPreservationPolicyService.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.business;

import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;
import org.springframework.stereotype.Service;

import ch.unige.solidify.rest.JoinResourceContainer;
import ch.unige.solidify.service.JoinResource2TiersService;
import ch.unige.solidify.specification.Join2TiersSpecification;

import ch.dlcm.controller.AdminController;
import ch.dlcm.model.display.OrganizationalUnitPreservationPolicyDTO;
import ch.dlcm.model.policies.OrganizationalUnitPreservationPolicy;
import ch.dlcm.model.policies.PreservationPolicy;
import ch.dlcm.model.settings.OrganizationalUnit;
import ch.dlcm.specification.OrganizationalUnitPreservationPolicySpecification;

@Service
@ConditionalOnBean(AdminController.class)
public class OrganizationalUnitPreservationPolicyService
        extends JoinResource2TiersService<OrganizationalUnit, PreservationPolicy, OrganizationalUnitPreservationPolicy> {

  @Override
  public Join2TiersSpecification<OrganizationalUnitPreservationPolicy> getJoinSpecification(
          OrganizationalUnitPreservationPolicy orgUnitPreservationPolicy) {
    return new OrganizationalUnitPreservationPolicySpecification(orgUnitPreservationPolicy);
  }

  @Override
  public PreservationPolicy getChildResource(OrganizationalUnitPreservationPolicy orgUnitPreservationPolicy) {
    return orgUnitPreservationPolicy.getPreservationPolicy();
  }

  @Override
  public String getChildPathFromJoinResource() {
    return OrganizationalUnitPreservationPolicy.PATH_TO_PRESERVATION_POLICY;
  }

  @Override
  public JoinResourceContainer<OrganizationalUnitPreservationPolicy> getChildDTO(
          OrganizationalUnitPreservationPolicy orgUnitPreservationPolicy) {
    return new OrganizationalUnitPreservationPolicyDTO(orgUnitPreservationPolicy);
  }

  @Override
  public OrganizationalUnitPreservationPolicy getEmptyJoinResourceObject() {
    return new OrganizationalUnitPreservationPolicy();
  }

  @Override
  public OrganizationalUnit getEmptyParentResourceObject() {
    return new OrganizationalUnit();
  }

  @Override
  public PreservationPolicy getEmptyChildResourceObject() {
    return new PreservationPolicy();
  }

  @Override
  public void setParentResource(OrganizationalUnitPreservationPolicy orgUnitPreservationPolicy, OrganizationalUnit orgUnit) {
    orgUnitPreservationPolicy.setOrganizationalUnit(orgUnit);
  }

  @Override
  public void setChildResource(OrganizationalUnitPreservationPolicy orgUnitPreservationPolicy, PreservationPolicy policy) {
    orgUnitPreservationPolicy.setPreservationPolicy(policy);
  }

}
