/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Admin - OrganizationalUnitSubmissionPolicyService.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.business;

import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;
import org.springframework.stereotype.Service;

import ch.unige.solidify.rest.JoinResourceContainer;
import ch.unige.solidify.service.JoinResource2TiersService;
import ch.unige.solidify.specification.Join2TiersSpecification;

import ch.dlcm.controller.AdminController;
import ch.dlcm.model.display.OrganizationalUnitSubmissionPolicyDTO;
import ch.dlcm.model.policies.OrganizationalUnitSubmissionPolicy;
import ch.dlcm.model.policies.SubmissionPolicy;
import ch.dlcm.model.settings.OrganizationalUnit;
import ch.dlcm.specification.OrganizationalUnitSubmissionPolicySpecification;

@Service
@ConditionalOnBean(AdminController.class)
public class OrganizationalUnitSubmissionPolicyService
        extends JoinResource2TiersService<OrganizationalUnit, SubmissionPolicy, OrganizationalUnitSubmissionPolicy> {

  @Override
  public Join2TiersSpecification<OrganizationalUnitSubmissionPolicy> getJoinSpecification(
          OrganizationalUnitSubmissionPolicy orgUnitSubmissionPolicy) {
    return new OrganizationalUnitSubmissionPolicySpecification(orgUnitSubmissionPolicy);
  }

  @Override
  public SubmissionPolicy getChildResource(OrganizationalUnitSubmissionPolicy orgUnitSubmissionPolicy) {
    return orgUnitSubmissionPolicy.getSubmissionPolicy();
  }

  @Override
  public String getChildPathFromJoinResource() {
    return OrganizationalUnitSubmissionPolicy.PATH_TO_SUBMISSION_POLICY;
  }

  @Override
  public JoinResourceContainer<OrganizationalUnitSubmissionPolicy> getChildDTO(
          OrganizationalUnitSubmissionPolicy orgUnitSubmissionPolicy) {
    return new OrganizationalUnitSubmissionPolicyDTO(orgUnitSubmissionPolicy);
  }

  @Override
  public OrganizationalUnitSubmissionPolicy getEmptyJoinResourceObject() {
    return new OrganizationalUnitSubmissionPolicy();
  }

  @Override
  public OrganizationalUnit getEmptyParentResourceObject() {
    return new OrganizationalUnit();
  }

  @Override
  public SubmissionPolicy getEmptyChildResourceObject() {
    return new SubmissionPolicy();
  }

  @Override
  public void setParentResource(OrganizationalUnitSubmissionPolicy orgUnitSubmissionPolicy, OrganizationalUnit orgUnit) {
    orgUnitSubmissionPolicy.setOrganizationalUnit(orgUnit);
  }

  @Override
  public void setChildResource(OrganizationalUnitSubmissionPolicy orgUnitSubmissionPolicy, SubmissionPolicy policy) {
    orgUnitSubmissionPolicy.setSubmissionPolicy(policy);
  }

}
