/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Admin - OrganizationalUnitsUserController.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.controller.admin;

import java.util.List;

import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import ch.unige.solidify.controller.ControllerWithHateoasHome;
import ch.unige.solidify.controller.SolidifyController;
import ch.unige.solidify.rest.RestCollection;
import ch.unige.solidify.security.UserPermissions;

import ch.dlcm.business.PersonService;
import ch.dlcm.controller.AdminController;
import ch.dlcm.model.security.Role;
import ch.dlcm.model.settings.Institution;
import ch.dlcm.repository.InstitutionPersonRoleRepository;
import ch.dlcm.rest.UrlPath;

@RestController
@UserPermissions
@ConditionalOnBean(AdminController.class)
@RequestMapping(UrlPath.ADMIN_AUTHORIZED_INSTITUTION)
public class InstitutionsUserController extends SolidifyController implements ControllerWithHateoasHome {
  private final PersonService personService;
  private final InstitutionPersonRoleRepository institutionPersonRoleRepository;

  public InstitutionsUserController(PersonService personService,
          InstitutionPersonRoleRepository institutionPersonRoleRepository) {
    this.personService = personService;
    this.institutionPersonRoleRepository = institutionPersonRoleRepository;
  }

  /**
   * @return a list of Institutions for which the authenticated user is manager
   */
  @GetMapping
  public HttpEntity<RestCollection<Institution>> listAll() {
    final String personId = this.personService.getPerson(SecurityContextHolder.getContext().getAuthentication()).getResId();
    final List<Institution> institutionList = this.institutionPersonRoleRepository.findByRoleAndPerson(Role.MANAGER_ID, personId);
    return new ResponseEntity<>(new RestCollection<>(institutionList), HttpStatus.OK);
  }
}
