/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Archival Storage - FileStorageServiceTest.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.test.storage;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.context.junit.jupiter.SpringExtension;

import ch.unige.solidify.config.SolidifyProperties;

import ch.dlcm.config.DLCMProperties;
import ch.dlcm.config.DLCMRepositoryDescription;
import ch.dlcm.service.MetadataService;
import ch.dlcm.storage.file.FileStorageService;

@ActiveProfiles({ "arc-file", "sec-noauth" })
@ExtendWith(SpringExtension.class)
class FileStorageServiceTest extends StorageServiceTest {

  @Test
  void bagItStorageTest() {
    this.runTests("faust-1072055880-aip-bagit", 10 + 8);
  }

  @BeforeEach
  public void setup() {
    super.setUp();
    // DLCM Properties
    this.config = new DLCMProperties(new SolidifyProperties());
    this.config.setHome(this.workingDir.toAbsolutePath().toString());
    this.config.getStorage().setUrl(this.storageDir.toAbsolutePath().toUri().toString());
    // DLCM Repository description
    this.repositoryDescription = new DLCMRepositoryDescription(this.gitInfoProperties);
    // Services
    this.metadataService = new MetadataService(
            this.fallbackLanguageRemoteService,
            this.fallbackOrgUnitRemoteService,
            this.fallbackAipRemoteService,
            this.historyService,
            this.config,
            this.repositoryDescription,
            this.messageService,
            this.gitInfoProperties,
            this.fallbackMetadataTypeRemoteService,
            this.fallbackPersonRemoteService,
            this.fallbackLicenseRemoteService,
            this.fallbackArchiveMetadataRemoteService,
            this.fallbackPreservationPolicyRemoteService,
            this.fallbackArchiveTypeResourceService);
    this.storageService = new FileStorageService(
            this.config,
            this.repositoryDescription,
            this.messageService,
            this.metadataService,
            this.fallbackOrgUnitRemoteService,
            this.fallbackArchiveThumbnailRemoteService,
            this.fallbackArchiveMetadataRemoteService);
  }

  @Test
  void zipStorageTest() {
    this.runTests("faust-1072055880-aip-zip", 10);
  }

}
