/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Client - NotificationClientService.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.service.admin;

import java.util.List;
import java.util.Map;

import org.springframework.core.env.Environment;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

import ch.unige.solidify.SolidifyClientApplication;
import ch.unige.solidify.service.ResourceClientService;
import ch.unige.solidify.util.SudoRestClientTool;

import ch.dlcm.model.notification.Notification;
import ch.dlcm.model.notification.NotificationStatus;
import ch.dlcm.rest.DLCMActionName;
import ch.dlcm.rest.ModuleName;
import ch.dlcm.rest.ResourceName;

@Service
public class NotificationClientService extends ResourceClientService<Notification> {

  private static final String NOTIFICATION_PATH = "/" + ResourceName.NOTIFICATION;

  public NotificationClientService(SudoRestClientTool restClientTool, Environment env) {
    super(SolidifyClientApplication.getApplicationPrefix(), ModuleName.ADMIN, NOTIFICATION_PATH, Notification.class, restClientTool, env);
  }

  @Override
  public List<Notification> findAll() {
    return this.findAllWithPagination(Notification.class);
  }

  public List<Notification> getInboxNotifications() {
    return this.doActionOnList(DLCMActionName.INBOX, Notification.class);
  }

  public List<Notification> getInboxNotifications(NotificationStatus notificationStatus) {
    String statusName = notificationStatus.toString();
    return this.docActionOnListWithParameter(DLCMActionName.INBOX, Notification.class, "notificationStatus", statusName);
  }

  public Notification getInboxNotification(String notificationId) {
    return this.doActionWithId(notificationId, DLCMActionName.INBOX);
  }

  public List<Notification> getSentNotifications() {
    return this.doActionOnList(DLCMActionName.SENT, Notification.class);
  }

  public Notification getSentNotification(String notificationId) {
    return this.doActionWithId(notificationId, DLCMActionName.SENT);
  }

  public Notification setProcessedStatus(String notificationId) {
    return this.applyAction(notificationId, DLCMActionName.SET_PROCESSED);
  }

  public Notification setRefusedStatus(String notificationId) {
    return this.applyAction(notificationId, DLCMActionName.SET_REFUSED);
  }

  public Notification setPendingStatus(String notificationId) {
    return this.applyAction(notificationId, DLCMActionName.SET_PENDING);
  }

  @Override
  public List<Notification> searchByProperties(Map<String, String> properties) {
    return this.searchByProperties(properties, Notification.class);
  }

  private Notification applyAction(String notificationId, String actionName) {
    final String url = this.getResourceUrl()
            + "/" + notificationId
            + "/" + actionName;
    final RestTemplate restTemplate = this.restClientTool.getClient(this.getResourceUrl());
    return restTemplate.postForObject(url, null, Notification.class);
  }
}
