/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Common - DLCMControllerAdvice.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.controlleradvvice;

import javax.servlet.http.HttpServletRequest;

import org.apache.catalina.connector.ClientAbortException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.boot.web.servlet.error.ErrorAttributes;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.bind.annotation.ExceptionHandler;

import ch.unige.solidify.controller.SolidifyControllerAdvice;
import ch.unige.solidify.service.MessageService;

@ControllerAdvice
public class DLCMControllerAdvice extends SolidifyControllerAdvice {
  private static final Logger log = LoggerFactory.getLogger(DLCMControllerAdvice.class);

  public DLCMControllerAdvice(ErrorAttributes errorAttributes, MessageService messageService) {
    super(errorAttributes, messageService);
  }

  // ClientAbortException belongs to Tomcat, handle it at project level since Solidify-Backend is container agnostic
  @ExceptionHandler(ClientAbortException.class)
  public void handleClientAbortException(ClientAbortException exception, HttpServletRequest request) {
    final String message = "ClientAbortException generated by request {} {} from remote address {} with X-FORWARDED-FOR {}";
    final String headerXFF = request.getHeader("X-FORWARDED-FOR");
    log.warn(message, request.getMethod(), request.getRequestURL(), request.getRemoteAddr(), headerXFF);
  }

}
