/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Data Management - PublicDIController.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.controller.datamgmt;

import java.util.List;

import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;
import org.springframework.data.domain.Pageable;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import ch.unige.solidify.business.IndexFieldAliasService;
import ch.unige.solidify.index.indexing.IndexingService;
import ch.unige.solidify.model.SearchRequest;
import ch.unige.solidify.rest.RestCollection;
import ch.unige.solidify.rest.SearchCondition;
import ch.unige.solidify.security.EveryonePermissions;
import ch.unige.solidify.security.TrustedUserPermissions;

import ch.dlcm.config.DLCMProperties;
import ch.dlcm.controller.DataMgmtController;
import ch.dlcm.model.index.ArchiveMetadata;
import ch.dlcm.rest.UrlPath;
import ch.dlcm.service.rest.propagate.PropagateOrganizationalUnitRemoteResourceService;

@EveryonePermissions
@RestController
@ConditionalOnBean(DataMgmtController.class)
@RequestMapping(UrlPath.DATA_MGMT_PUBLIC_DI)
public class PublicDIController extends DLCMIndexingController {

  private final String publicIndex;

  public PublicDIController(
          IndexingService<ArchiveMetadata> noSqlResourceService,
          IndexFieldAliasService indexFieldAliasService,
          PropagateOrganizationalUnitRemoteResourceService organizationalUnitRemoteResourceService,
          DLCMProperties config) {
    super(noSqlResourceService, indexFieldAliasService, organizationalUnitRemoteResourceService);
    this.publicIndex = config.getIndexing().getIndexName();
  }

  // **********************************
  // Override methods to apply security
  // **********************************

  @Override
  public HttpEntity<ArchiveMetadata> create(@RequestBody ArchiveMetadata t) {
    return super.create(t);
  }

  @Override
  public HttpEntity<ArchiveMetadata> get(@PathVariable String id) {
    return super.get(id);
  }

  @Override
  public HttpEntity<RestCollection<ArchiveMetadata>> list(@ModelAttribute ArchiveMetadata search, Pageable pageable) {
    return super.list(search, pageable);
  }

  @TrustedUserPermissions
  @Override
  public HttpEntity<ArchiveMetadata> update(@PathVariable String id, @RequestBody ArchiveMetadata t2) {
    return super.update(id, t2);
  }

  @TrustedUserPermissions
  @Override
  public HttpStatus delete(@PathVariable String id) {
    return super.delete(id);
  }

  @TrustedUserPermissions
  @Override
  public HttpStatus deleteList(@RequestBody String[] ids) {
    return super.deleteList(ids);
  }

  @Override
  public HttpEntity<RestCollection<ArchiveMetadata>> search(@RequestParam(defaultValue = "*") String query,
          @RequestParam(required = false) String field, @RequestParam(required = false) String from, @RequestParam(required = false) String to,
          @RequestParam(required = false) String set, @RequestParam(required = false) String orgUnit,
          @RequestParam(required = false) String conditions, Pageable pageable) {
    return super.search(query, field, from, to, set, orgUnit, conditions, pageable);
  }

  @Override
  public HttpEntity<RestCollection<ArchiveMetadata>> searchPost(@RequestBody SearchRequest searchRequest) {
    return super.searchPost(searchRequest);
  }
  // **********************************

  @Override
  protected String getIndex() {
    return this.publicIndex;
  }

  /**
   * Override to not add conditions: as the query is made on public index there is no filtering on
   * authorized org units.
   *
   * @param searchConditions
   */
  @Override
  protected void addSpecificCondition(List<SearchCondition> searchConditions) {
  }
}
