/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Integration Tests - DLCMTestConstants.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.test;

import static org.junit.jupiter.api.Assertions.fail;

import java.util.concurrent.ThreadLocalRandom;

import ch.unige.solidify.util.SolidifyTime;

import ch.dlcm.DLCMConstants;

public class DLCMTestConstants {
  public final static int TRIES = 180;
  public final static String PRESERVATION_POLICY_TEST_NAME = "PreservationPolicy for testing";
  public final static String SUBMISSION_POLICY_TEST_NAME = "SubmissionPolicy for testing";

  public enum OrganizationalUnitStatus {
    CLOSED, OPEN
  }

  public enum PersistenceMode {
    PERMANENT, TEMPORARY
  }

  public static void failAfterNAttempt(int count, String failMessage) {
    DLCMTestConstants.failAfterNAttempt(count, 1, failMessage);
  }

  public static void failAfterNAttempt(int count, int waitInSeconds, String failMessage) {
    if (count > DLCMTestConstants.TRIES) {
      fail(failMessage);
    }
    SolidifyTime.waitInSeconds(waitInSeconds);
  }

  public static String getNameWithPermanentLabel(String name) {
    return DLCMConstants.PERMANENT_TEST_DATA_LABEL + name;
  }

  public static String getRandomName(String name) {
    return name + ThreadLocalRandom.current().nextInt();
  }

  public static String getRandomNameWithTemporaryLabel(String name) {
    return DLCMConstants.TEMPORARY_TEST_DATA_LABEL + DLCMTestConstants.getRandomName(name);
  }

  public static String getRandomOrcId() {
    String orcId = "";
    for (int i = 0; i < 4; i++) {
      for (int j = 0; j < 4; j++) {
        orcId = orcId + ThreadLocalRandom.current().nextInt(10);
      }
      orcId = orcId + "-";
    }
    return orcId.substring(0, orcId.length() - 1);
  }
}
