/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Integration Tests - AbstractInstitutionUserIT.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.test.admin;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNotNull;

import java.io.IOException;
import java.nio.file.Path;
import java.nio.file.Paths;

import org.junit.jupiter.api.BeforeAll;
import org.springframework.core.io.ClassPathResource;

import ch.dlcm.model.settings.Institution;
import ch.dlcm.model.settings.Person;
import ch.dlcm.test.DLCMTestConstants;

abstract class AbstractInstitutionUserIT extends AbstractAdminIT {

  protected abstract String role();

  @Override
  protected void setUser() {
    this.restClientTool.sudoUser();
  }

  @BeforeAll
  void setUp() {
    this.restClientTool.sudoAdmin();
    this.getOrCreatePermanentInstitution(DLCMTestConstants.PersistenceMode.PERMANENT, this.role());
    this.restClientTool.exitSudo();
  }

  @Override
  protected void deleteFixtures() {
    this.restClientTool.sudoAdmin();
    this.clearInstitutionsFixtures();
    this.restClientTool.exitSudo();
  }

  void downloadLogoShouldSuccessTest() throws IOException {
    final String logoName = "geneve.jpg";
    final ClassPathResource logoToUpload = new ClassPathResource(logoName);
    final Long sizeLogo = logoToUpload.contentLength();

    this.restClientTool.sudoAdmin();
    final Institution institution1 = this.createRemoteInstitutionAsAdminWithCurrentUserWithRole("DLCM Institution #LOGO");
    Institution institution2 = this.institutionService.uploadLogo(institution1.getResId(), logoToUpload);

    final Institution refetchedInstitution = this.institutionService.findOne(institution2.getResId());
    assertEquals(refetchedInstitution.getImage().getFileName(), logoName);
    assertEquals(refetchedInstitution.getImage().getFileSize(), sizeLogo);
    this.restClientTool.exitSudo();

    Path path = Paths.get(System.getProperty("java.io.tmpdir"), "download.tmp");
    this.institutionService.downloadLogo(refetchedInstitution.getResId(), path);
    assertNotNull(path.toFile());
    assertEquals(Long.valueOf(path.toFile().length()), refetchedInstitution.getImage().getFileSize());
  }

  protected Institution createRemoteInstitution(String description) {
    Institution institution = new Institution();
    institution.setName(DLCMTestConstants.getRandomNameWithTemporaryLabel(description));
    institution = this.institutionService.create(institution);
    return institution;
  }

  protected Institution createRemoteInstitutionAsAdminWithCurrentUserWithRole(String description) {
    this.restClientTool.sudoAdmin();
    Institution institution = this.createRemoteInstitution(description);
    final Person person = this.getOrCreateRemotePerson();
    this.enforcePersonHasRoleInInstitution(institution, person, this.role());
    this.restClientTool.exitSudo();
    return institution;
  }

  protected Institution createRemoteInstitutionAsAdmin(String description) {
    this.restClientTool.sudoAdmin();
    Institution institution = this.createRemoteInstitution(description);
    this.restClientTool.exitSudo();
    return institution;
  }

}
