/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Integration Tests - AbstractNotificationIT.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.test.admin;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertTrue;

import java.util.List;

import ch.dlcm.DLCMConstants;
import ch.dlcm.model.notification.Notification;
import ch.dlcm.model.notification.NotificationStatus;
import ch.dlcm.model.notification.NotificationType;
import ch.dlcm.model.security.User;
import ch.dlcm.model.settings.OrganizationalUnit;
import ch.dlcm.test.DLCMTestConstants;

public abstract class AbstractNotificationIT extends AbstractAdminIT {

  private static final String DEFAULT_MESSAGE = " A message";

  Notification createLocalNotification(NotificationType notificationType, OrganizationalUnit notifiedOrgUnit) {
    return this.createLocalNotification(notificationType, notifiedOrgUnit, DEFAULT_MESSAGE);
  }

  Notification createLocalNotification(NotificationType notificationType, OrganizationalUnit notifiedOrgUnit, String message) {
    return this.createLocalNotification(notificationType, notifiedOrgUnit, message, null);
  }

  Notification createLocalNotification(NotificationType notificationType, OrganizationalUnit notifiedOrgUnit, String message, String objectId) {
    User emitter = this.userService.getAuthenticatedUser();
    Notification localNotification = new Notification();
    localNotification.setEmitter(emitter);
    localNotification.setMessage(DLCMConstants.TEMPORARY_TEST_DATA_LABEL + " " + message);
    localNotification.setObjectId(objectId);
    localNotification.setNotificationType(notificationType);
    localNotification.setNotifiedOrgUnit(notifiedOrgUnit);
    return localNotification;
  }

  Notification createRemoteNotification(Notification localNotification) {
    return this.notificationService.create(localNotification);
  }

  OrganizationalUnit createOrgUnitForCurrentUser(String role) {
    final User currentUser = this.userService.getAuthenticatedUser();
    final OrganizationalUnit localNotifiedOrgUnit = new OrganizationalUnit();
    localNotifiedOrgUnit.setName(DLCMTestConstants.getRandomNameWithTemporaryLabel("OrgUnit"));
    final OrganizationalUnit remoteNotifiedOrgUnit = this.organizationalUnitService.create(localNotifiedOrgUnit);
    this.enforcePersonHasRoleInOrgUnit(remoteNotifiedOrgUnit, currentUser.getPerson(), role);
    return remoteNotifiedOrgUnit;
  }

  Notification createRemoteNotificationWithoutOrgUnit(NotificationType notificationType, String objectId) {
    final Notification localNotification = this.createLocalNotification(notificationType, null, null, objectId);
    return this.notificationService.create(localNotification);
  }

  void inboxTest(NotificationType notificationType, OrganizationalUnit notifiedOrgUnit, boolean shouldBeInInbox) {
    this.inboxTest(notificationType, notifiedOrgUnit, null, shouldBeInInbox);
  }

  void inboxTest(NotificationType notificationType, OrganizationalUnit notifiedOrgUnit, String objectId, boolean shouldBeInInbox) {
    final Notification localNotification = this.createLocalNotification(notificationType, notifiedOrgUnit, DEFAULT_MESSAGE, objectId);
    final Notification remoteNotification = this.createRemoteNotification(localNotification);
    final Notification inboxNotification = this.notificationService.getInboxNotification(remoteNotification.getResId());
    assertEquals(inboxNotification != null, shouldBeInInbox, "Sent notification should be in inbox");
    if (shouldBeInInbox) {
      final List<Notification> pendingNotifications = this.notificationService.getInboxNotifications(NotificationStatus.PENDING);
      assertTrue(pendingNotifications.size() >= 1, "There should be at least one notification in the pending status");
      for (Notification notification : pendingNotifications) {
        assertEquals(NotificationStatus.PENDING, notification.getNotificationStatus(),
                "All notifications filtered by PENDING status should be in the pending status");
      }
    }
  }

  @Override
  protected void deleteFixtures() {
    this.clearNotificationFixtures();
    this.clearOrganizationalUnitFixtures();
  }

}
