/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Integration Tests - ArchiveAclAsAdminIT.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.test.admin;

import static org.junit.jupiter.api.Assertions.assertDoesNotThrow;
import static org.junit.jupiter.api.Assertions.assertEquals;

import java.time.OffsetDateTime;
import java.time.ZoneOffset;
import java.util.List;
import java.util.Map;

import org.junit.jupiter.api.Test;

import ch.unige.solidify.model.index.IndexMetadata;

import ch.dlcm.DLCMConstants;
import ch.dlcm.model.oais.ArchivalInfoPackage;
import ch.dlcm.model.security.User;
import ch.dlcm.model.settings.ArchiveACL;
import ch.dlcm.model.settings.OrganizationalUnit;
import ch.dlcm.test.DLCMTestConstants;

class ArchiveAclAsAdminIT extends AbstractAdminIT {

  protected String archiveName = DLCMConstants.ARCHIVAL_UNIT_LABEL + " 1";

  @Override
  protected void setUser() {
    this.restClientTool.sudoAdmin();
  }

  @Test
  void creationTest() {
    // Create an archive acl
    final ArchiveACL archiveACL = new ArchiveACL();

    // Obtain aip of archive already created by init data test
    final List<ArchivalInfoPackage> aipList = this.aipService.searchByProperties(Map.of("info.name", this.archiveName));
    if (!aipList.isEmpty()) {
      final String aipId = aipList.get(0).getResId();
      IndexMetadata IndexMetadata = this.metadataService.findOne(aipId);
      String orgUnitId = (String) (IndexMetadata.getMetadata(DLCMConstants.AIP_ORGA_UNIT));
      OrganizationalUnit organizationalUnit = this.organizationalUnitService.findOne(orgUnitId);

      // obtain connected user
      User user = this.userService.getAuthenticatedUser();
      archiveACL.setAipId(aipId);
      archiveACL.setUser(user);
      archiveACL.setOrganizationalUnit(organizationalUnit);
      ArchiveACL archiveACL1 = this.archiveACLService.create(archiveACL);

      // Test the creation
      assertEquals(archiveACL.getUser().getResId(), archiveACL1.getUser().getResId());
      assertEquals(archiveACL.getOrganizationalUnit().getResId(), archiveACL1.getOrganizationalUnit().getResId());
      assertEquals(archiveACL.getAipId(), archiveACL1.getAipId());
    }
  }

  @Test
  void updateTest() {
    // Create archiveACL
    ArchiveACL archiveACL = new ArchiveACL();

    // Obtain aip of archive already created by init data test
    final List<ArchivalInfoPackage> aipList = this.aipService.searchByProperties(Map.of("info.name", this.archiveName));
    if (!aipList.isEmpty()) {
      final String aipId = aipList.get(0).getResId();
      IndexMetadata IndexMetadata = this.metadataService.findOne(aipId);
      String orgUnitId = (String) (IndexMetadata.getMetadata(DLCMConstants.AIP_ORGA_UNIT));
      OrganizationalUnit organizationalUnit = this.organizationalUnitService.findOne(orgUnitId);

      // Obtain connected user
      User user = this.userService.getAuthenticatedUser();
      archiveACL.setAipId(aipId);
      archiveACL.setUser(user);
      archiveACL.setOrganizationalUnit(organizationalUnit);
      archiveACL = this.archiveACLService.create(archiveACL);

      // Update archiveACL
      archiveACL.setExpiration(OffsetDateTime.now(ZoneOffset.UTC).plusYears(1));
      this.archiveACLService.update(archiveACL.getResId(), archiveACL);
      final ArchiveACL archiveACL1 = this.archiveACLService.findOne(archiveACL.getResId());

      // Test the update
      assertEquals(archiveACL.getUser().getResId(), archiveACL1.getUser().getResId());
      this.assertEqualsWithoutNanoSeconds(archiveACL.getExpiration(), archiveACL1.getExpiration());
      assertEquals(archiveACL.getAipId(), archiveACL1.getAipId());
    }
  }

  @Test
  void listTest() {
    OrganizationalUnit organizationalUnit = this.getOrCreateOrganizationalUnit(DLCMTestConstants.PersistenceMode.PERMANENT,
            DLCMTestConstants.OrganizationalUnitStatus.OPEN);
    assertDoesNotThrow(() -> this.archiveACLService.searchByProperties(Map.of("organizationalUnit.resId", organizationalUnit.getResId())));
  }

  @Override
  protected void deleteFixtures() {
    this.clearArchiveAclFixtures();
  }

  private void clearArchiveAclFixtures() {
    // clear all archive-acl by this user to the archive used in these tests
    final List<ArchivalInfoPackage> aipList = this.aipService.searchByProperties(Map.of("info.name", this.archiveName));
    if (!aipList.isEmpty()) {
      final String aipId = aipList.get(0).getResId();
      IndexMetadata IndexMetadata = this.metadataService.findOne(aipId);
      String orgUnitId = (String) (IndexMetadata.getMetadata(DLCMConstants.AIP_ORGA_UNIT));

      // obtain connected user
      User user = this.userService.getAuthenticatedUser();

      List<ArchiveACL> archiveAclList = this.archiveACLService.searchByProperties(Map.of("aipId", aipId,
              "organizationalUnit.resId", orgUnitId,
              "user.resId", user.getResId()));
      for (ArchiveACL archiveACL : archiveAclList) {
        this.archiveACLService.delete(archiveACL.getResId());
      }
    }

  }
}
