/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Integration Tests - ArchiveAclAsUserIT.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.test.admin;

import static org.junit.jupiter.api.Assertions.assertThrows;

import java.util.Map;

import org.junit.jupiter.api.Test;
import org.springframework.web.client.HttpClientErrorException;

import ch.unige.solidify.auth.model.AuthApplicationRole;

import ch.dlcm.model.security.Role;
import ch.dlcm.model.settings.ArchiveACL;
import ch.dlcm.model.settings.OrganizationalUnit;
import ch.dlcm.test.DLCMTestConstants;
import ch.dlcm.test.DLCMTestConstants.OrganizationalUnitStatus;

class ArchiveAclAsUserIT extends AbstractAdminIT {

  @Override
  protected void setUser() {
    this.restClientTool.sudoUser();
  }

  @Test
  void creationWithoutAipIdTest() {
    this.restClientTool.sudoAdmin();
    OrganizationalUnit managerOrgUnit = this.getOrCreateOrganizationalUnit(DLCMTestConstants.PersistenceMode.PERMANENT, OrganizationalUnitStatus.OPEN, Role.MANAGER_ID,
            AuthApplicationRole.USER_ID);
    this.restClientTool.exitSudo();
    ArchiveACL acl = new ArchiveACL();
    acl.setOrganizationalUnit(managerOrgUnit);
    assertThrows(HttpClientErrorException.Forbidden.class,
            () -> this.archiveACLService.create(acl));
  }

  @Test
  void creationWithMissingAip() {
    OrganizationalUnit orgUnitForNoOne = this.getOrgUnitForNoOne(DLCMTestConstants.PersistenceMode.PERMANENT);
    ArchiveACL acl = new ArchiveACL();
    acl.setOrganizationalUnit(orgUnitForNoOne);
    acl.setAipId("000");
    assertThrows(HttpClientErrorException.Forbidden.class,
            () -> this.archiveACLService.create(acl));
  }

  @Test
  void failUpdateTest() {
    OrganizationalUnit orgUnitForNoOne = this.getOrgUnitForNoOne(DLCMTestConstants.PersistenceMode.PERMANENT);
    ArchiveACL acl = new ArchiveACL();
    acl.setOrganizationalUnit(orgUnitForNoOne);
    assertThrows(HttpClientErrorException.NotFound.class,
            () -> this.archiveACLService.update(acl.getResId(), acl));
  }

  @Test
  void failDeleteTest() {
    OrganizationalUnit orgUnitForNoOne = this.getOrgUnitForNoOne(DLCMTestConstants.PersistenceMode.PERMANENT);
    ArchiveACL acl = new ArchiveACL();
    acl.setOrganizationalUnit(orgUnitForNoOne);
    assertThrows(HttpClientErrorException.NotFound.class,
            () -> this.archiveACLService.delete(acl.getResId()));
  }

  @Test
  void failListTest() {
    OrganizationalUnit orgUnitForNoOne = this.getOrgUnitForNoOne(DLCMTestConstants.PersistenceMode.PERMANENT);
    ArchiveACL acl = new ArchiveACL();
    acl.setOrganizationalUnit(orgUnitForNoOne);
    assertThrows(HttpClientErrorException.Forbidden.class,
            () -> this.archiveACLService.searchByProperties(Map.of("organizationalUnit.resId", orgUnitForNoOne.getResId())));
  }

  @Override
  protected void deleteFixtures() {
    // Do nothing
  }
}
