/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Integration Tests - DisseminationPolicyIT.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.test.admin;

import static org.junit.jupiter.api.Assertions.assertEquals;

import org.junit.jupiter.api.Test;

import ch.dlcm.model.policies.DisseminationPolicy;
import ch.dlcm.test.DLCMTestConstants;

class DisseminationPolicyIT extends AbstractAdminIT {

  @Override
  protected void setUser() {
    this.restClientTool.sudoAdmin();
  }

  @Test
  void creationTest() {
    // Create a policy
    final DisseminationPolicy policy1 = new DisseminationPolicy();
    policy1.setName(DLCMTestConstants.getRandomNameWithTemporaryLabel("dissemination policy "));
    policy1.setType(DisseminationPolicy.DisseminationPolicyType.WEB);
    policy1.setDestinationServer("http://server.com");
    final DisseminationPolicy policy2 = this.disseminationPolicyService.create(policy1);

    // Test the creation
    assertEquals(policy1.getName(), policy2.getName());
    assertEquals(policy1.getType(), policy2.getType());
  }

  @Test
  void updateTest() {
    // Create a policy
    DisseminationPolicy policy1 = new DisseminationPolicy();
    policy1.setName(DLCMTestConstants.getRandomNameWithTemporaryLabel("A"));
    policy1.setType(DisseminationPolicy.DisseminationPolicyType.WEB);
    policy1.setDestinationServer("http://server.com");
    policy1 = this.disseminationPolicyService.create(policy1);

    // Update the institution
    policy1.setName(DLCMTestConstants.getRandomNameWithTemporaryLabel("B"));
    policy1.setType(DisseminationPolicy.DisseminationPolicyType.IIIF);
    this.disseminationPolicyService.update(policy1.getResId(), policy1);
    final DisseminationPolicy policy2 = this.disseminationPolicyService.findOne(policy1.getResId());

    // Test the update
    assertEquals(policy1.getName(), policy2.getName());
    assertEquals(policy1.getType(), policy2.getType());
    this.assertEqualsWithoutNanoSeconds(policy1.getCreationTime(), policy2.getCreationTime());
  }

  @Override
  protected void deleteFixtures() {
    this.clearOrganizationalUnitFixtures();
    this.clearDisseminationPolicyFixtures();
  }

}
