/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Integration Tests - NotificationAsUserIT.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.test.admin;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertThrows;

import org.junit.jupiter.api.Test;
import org.springframework.web.client.HttpClientErrorException;

import ch.dlcm.DLCMConstants;
import ch.dlcm.model.notification.Notification;
import ch.dlcm.model.notification.NotificationStatus;
import ch.dlcm.model.notification.NotificationType;
import ch.dlcm.model.settings.OrganizationalUnit;
import ch.dlcm.test.DLCMTestConstants;
import ch.dlcm.test.DLCMTestConstants.OrganizationalUnitStatus;

class NotificationAsUserIT extends AbstractNotificationIT {

  @Test
  void inboxTest() {
    this.inboxTest(NotificationType.CREATE_ORGUNIT_REQUEST, null, DLCMConstants.TEMPORARY_TEST_DATA_LABEL + " New org unit name", true);
  }

  @Test
  void newOrgUnitNotificationWithoutObject() {
    HttpClientErrorException e = assertThrows(HttpClientErrorException.class,
            () -> this.inboxTest(NotificationType.CREATE_ORGUNIT_REQUEST, null, false));
    this.assertFieldContainsValidationErrorMessage(e, "objectId",
            "Notifications of requests to create new organizational units require an organizational unit name");
  }

  @Override
  protected void deleteFixtures() {
    // Do nothing
  }

  @Test
  void changeStatusAllowedTest() {
    OrganizationalUnit notifiedOrgUnit = this.getOrCreateOrganizationalUnit(DLCMTestConstants.PersistenceMode.PERMANENT, OrganizationalUnitStatus.OPEN, "MANAGER");

    final Notification localNotification = this.createLocalNotification(NotificationType.ACCESS_DATASET_REQUEST, notifiedOrgUnit);
    Notification remoteNotification = this.createRemoteNotification(localNotification);
    assertEquals(NotificationStatus.PENDING, remoteNotification.getNotificationStatus(), "Initial notification status should be PENDING");

    final String notificationId = remoteNotification.getResId();
    this.notificationService.setProcessedStatus(notificationId);
    remoteNotification = this.notificationService.getInboxNotification(notificationId);
    assertEquals(NotificationStatus.PROCESSED, remoteNotification.getNotificationStatus(), "Notification status should be PROCESSED");

    this.notificationService.setRefusedStatus(notificationId);
    remoteNotification = this.notificationService.getInboxNotification(notificationId);
    assertEquals(NotificationStatus.REFUSED, remoteNotification.getNotificationStatus(), "Notification status should be REFUSED");

    this.notificationService.setPendingStatus(notificationId);
    remoteNotification = this.notificationService.getInboxNotification(notificationId);
    assertEquals(NotificationStatus.PENDING, remoteNotification.getNotificationStatus(), "Notification status should be PENDING");

  }

  @Test
  void changeStatusTest() {
    OrganizationalUnit notifiedOrgUnit = this.getOrgUnitForNoOne(DLCMTestConstants.PersistenceMode.PERMANENT);
    final Notification localNotification = this.createLocalNotification(NotificationType.ACCESS_DATASET_REQUEST, notifiedOrgUnit);
    Notification remoteNotification = this.createRemoteNotification(localNotification);
    assertEquals(NotificationStatus.PENDING, remoteNotification.getNotificationStatus(), "Initial notification status should be PENDING");

    this.notificationService.setProcessedStatus(remoteNotification.getResId());
    remoteNotification = this.notificationService.getInboxNotification(remoteNotification.getResId());
    assertEquals(NotificationStatus.PROCESSED, remoteNotification.getNotificationStatus(), "Notification status should be PROCESSED");

    this.notificationService.setRefusedStatus(remoteNotification.getResId());
    remoteNotification = this.notificationService.getInboxNotification(remoteNotification.getResId());
    assertEquals(NotificationStatus.REFUSED, remoteNotification.getNotificationStatus(), "Notification status should be REFUSED");

    this.notificationService.setPendingStatus(remoteNotification.getResId());
    remoteNotification = this.notificationService.getInboxNotification(remoteNotification.getResId());
    assertEquals(NotificationStatus.PENDING, remoteNotification.getNotificationStatus(), "Notification status should be PENDING");
  }
}
