/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Integration Tests - SubmissionAgreementIT.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.test.admin;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertFalse;
import static org.junit.jupiter.api.Assertions.assertNotNull;
import static org.junit.jupiter.api.Assertions.assertThrows;
import static org.junit.jupiter.api.Assertions.assertTrue;

import java.util.List;

import org.junit.jupiter.api.Test;
import org.springframework.web.client.HttpClientErrorException;

import ch.dlcm.model.settings.SubmissionAgreement;
import ch.dlcm.test.DLCMTestConstants;

class SubmissionAgreementIT extends AbstractAdminIT {

  @Override
  protected void setUser() {
    this.restClientTool.sudoAdmin();
  }

  @Test
  void creationTest() {
    // Create a submission agreement
    SubmissionAgreement expectedSubmissionAgreement = new SubmissionAgreement();
    expectedSubmissionAgreement.setTitle(DLCMTestConstants.getRandomNameWithTemporaryLabel("DLCM project"));
    expectedSubmissionAgreement.setDescription("Swiss Research Data Preservation #CREATE");
    expectedSubmissionAgreement.setVersion(DLCMTestConstants.getRandomNameWithTemporaryLabel("1"));
    expectedSubmissionAgreement = this.submissionAgreementService.create(expectedSubmissionAgreement);
    final SubmissionAgreement actualSubmissionAgreement = this.submissionAgreementService.findOne(expectedSubmissionAgreement.getResId());

    // Test the creation
    assertNotNull(expectedSubmissionAgreement.getResId());
    assertEquals(expectedSubmissionAgreement.getTitle(), actualSubmissionAgreement.getTitle());
    assertEquals(expectedSubmissionAgreement.getDescription(), actualSubmissionAgreement.getDescription());
    this.assertEqualsWithoutNanoSeconds(expectedSubmissionAgreement.getCreationTime(), actualSubmissionAgreement.getCreationTime());
  }

  @Test
  void deleteTest() {
    // Create a new submission agreement
    SubmissionAgreement expectedSubmissionAgreement = new SubmissionAgreement();
    expectedSubmissionAgreement.setTitle(DLCMTestConstants.getRandomNameWithTemporaryLabel("Deposit "));
    expectedSubmissionAgreement.setVersion(DLCMTestConstants.getRandomNameWithTemporaryLabel("1"));
    expectedSubmissionAgreement = this.submissionAgreementService.create(expectedSubmissionAgreement);

    // Delete
    final String resId = expectedSubmissionAgreement.getResId();
    this.submissionAgreementService.delete(resId);
    assertThrows(HttpClientErrorException.NotFound.class, () -> this.submissionAgreementService.findOne(resId));
  }

  @Test
  void findAllTest() {
    // Create a new submission agreement
    final SubmissionAgreement expectedSubmissionAgreement = new SubmissionAgreement();
    expectedSubmissionAgreement.setTitle(DLCMTestConstants.getRandomNameWithTemporaryLabel("Deposit "));
    expectedSubmissionAgreement.setVersion(DLCMTestConstants.getRandomNameWithTemporaryLabel("1"));
    this.submissionAgreementService.create(expectedSubmissionAgreement);

    // Find all
    final List<SubmissionAgreement> actualSubmissionAgreements = this.submissionAgreementService.findAll();

    // Test at least
    assertFalse(actualSubmissionAgreements.isEmpty());
    assertTrue(actualSubmissionAgreements.size() >= 1);
  }

  @Test
  void updateTest() {
    // Create a submission agreement
    SubmissionAgreement expectedSubmissionAgreement = new SubmissionAgreement();
    expectedSubmissionAgreement.setTitle(DLCMTestConstants.getRandomNameWithTemporaryLabel("A"));
    expectedSubmissionAgreement.setVersion(DLCMTestConstants.getRandomNameWithTemporaryLabel("1"));
    expectedSubmissionAgreement = this.submissionAgreementService.create(expectedSubmissionAgreement);

    // Update the submission agreement
    expectedSubmissionAgreement.setTitle(DLCMTestConstants.getRandomNameWithTemporaryLabel("B"));
    expectedSubmissionAgreement.setVersion(DLCMTestConstants.getRandomNameWithTemporaryLabel("2"));
    final SubmissionAgreement actualSubmissionAgreement = this.submissionAgreementService
            .update(expectedSubmissionAgreement.getResId(), expectedSubmissionAgreement);

    // Test the update
    assertNotNull(actualSubmissionAgreement);
    assertEquals(expectedSubmissionAgreement.getTitle(), actualSubmissionAgreement.getTitle());
    this.assertEqualsWithoutNanoSeconds(expectedSubmissionAgreement.getCreationTime(), actualSubmissionAgreement.getCreationTime());
    assertEquals(expectedSubmissionAgreement.getVersion(), actualSubmissionAgreement.getVersion());

  }

  @Override
  protected void deleteFixtures() {
    this.clearSubmissionAgreementFixtures();
  }
}
