/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Integration Tests - AbstractCheckDemoIT.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.test.demo;

import static org.junit.jupiter.api.Assertions.assertNotNull;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ch.dlcm.model.preingest.Deposit;
import ch.dlcm.model.settings.OrganizationalUnit;
import ch.dlcm.service.archivalstorage.AIPClientServiceSecondStorage;

public abstract class AbstractCheckDemoIT extends AbstractPreingestDemoIT {

  private static final Logger log = LoggerFactory.getLogger(AbstractCheckDemoIT.class);

  private AIPClientServiceSecondStorage aipService2;

  public AbstractCheckDemoIT(AIPClientServiceSecondStorage aipService2) {
    this.aipService2 = aipService2;
  }

  protected void checkDeposits(String orgUnitName) {
    final OrganizationalUnit orgUnit = this.getOrgUnit(orgUnitName);
    assertNotNull(orgUnit, "Cannot find orgUnit: " + orgUnitName);
    log.info("Checking deposits");
    for (final Deposit deposit : this.depositService.findAll()) {
      if (!deposit.getOrganizationalUnitId().equals(orgUnit.getResId())) {
        continue;
      }
      checkDeposit(deposit);
    }
  }

  protected void checkSIP(String orgUnitName) {
    final OrganizationalUnit orgUnit = this.getOrgUnit(orgUnitName);
    assertNotNull(orgUnit, "Cannot find orgUnit: " + orgUnitName);
    log.info("Checking SIP");
    for (final Deposit deposit : this.depositService.findAll()) {
      if (!deposit.getOrganizationalUnitId().equals(orgUnit.getResId())) {
        continue;
      }
      // Check
      checkDeposit(deposit);
      checkSIP(deposit);
    }
  }

  protected void checkAIPOnMainStorage(String orgUnitName) {
    final OrganizationalUnit orgUnit = this.getOrgUnit(orgUnitName);
    assertNotNull(orgUnit, "Cannot find orgUnit: " + orgUnitName);
    log.info("Checking AIP [Main storage]");
    for (final Deposit deposit : this.depositService.findAll()) {
      if (!deposit.getOrganizationalUnitId().equals(orgUnit.getResId())) {
        continue;
      }
      // Check
      checkDeposit(deposit);
      checkAIP(deposit, this.aipService);
    }
  }

  protected void checkAIPOnReplicaStorage(String orgUnitName) {
    final OrganizationalUnit orgUnit = this.getOrgUnit(orgUnitName);
    assertNotNull(orgUnit, "Cannot find orgUnit: " + orgUnitName);
    log.info("Checking AIP [Replica storage]");
    for (final Deposit deposit : this.depositService.findAll()) {
      if (!deposit.getOrganizationalUnitId().equals(orgUnit.getResId())) {
        continue;
      }
      // Check
      checkDeposit(deposit);
      checkAIP(deposit, this.aipService2);
    }
  }

  protected void checkArchives(String orgUnitName) {
    final OrganizationalUnit orgUnit = this.getOrgUnit(orgUnitName);
    assertNotNull(orgUnit, "Cannot find orgUnit: " + orgUnitName);
    log.info("Checking archives");
    for (final Deposit deposit : this.depositService.findAll()) {
      if (!deposit.getOrganizationalUnitId().equals(orgUnit.getResId())) {
        continue;
      }
      // Check
      checkDeposit(deposit);
      checkArchive(deposit);
    }
  }
}
