/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Integration Tests - AbstractDepositWithRoleIT.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.test.preingest;

import org.junit.jupiter.api.BeforeAll;

import ch.unige.solidify.auth.model.AuthApplicationRole;

import ch.dlcm.model.preingest.Deposit;
import ch.dlcm.model.settings.OrganizationalUnit;
import ch.dlcm.test.DLCMTestConstants;

abstract class AbstractDepositWithRoleIT extends AbstractPreingestIT {

  protected abstract String role();

  protected boolean inheritedRole() {
    return false;
  }

  @Override
  protected void setUser() {
    this.restClientTool.sudoUser();
  }

  @BeforeAll
  void setUp() {
    this.restClientTool.sudoAdmin();
    this.getOrCreateOrganizationalUnit(DLCMTestConstants.PersistenceMode.PERMANENT, DLCMTestConstants.OrganizationalUnitStatus.OPEN,
            this.role(), AuthApplicationRole.USER_ID, this.inheritedRole());
    this.restClientTool.exitSudo();
  }

  protected Deposit createRemoteDepositForCurrentRole() {
    return this.createRemoteDepositForCurrentRole(DEFAULT_TITLE);
  }

  protected Deposit createRemoteDepositForCurrentRole(String title) {
    return this.createRemoteDeposit(DLCMTestConstants.PersistenceMode.TEMPORARY, title, DEFAULT_DESCRIPTION, this.role(), this.inheritedRole());
  }

  protected Deposit createLocalDepositForCurrentRole() {
    return this.createLocalDepositForCurrentRole(DEFAULT_TITLE);
  }

  protected Deposit createLocalDepositForCurrentRole(String title) {
    return this.createLocalDeposit(DLCMTestConstants.PersistenceMode.TEMPORARY, title, DEFAULT_DESCRIPTION, this.role(), this.inheritedRole());
  }

  protected OrganizationalUnit getPermanentOrganizationalUnitWithRole() {
    return this.getPermanentOrganizationalUnitWithRole(DLCMTestConstants.OrganizationalUnitStatus.OPEN);
  }

  protected OrganizationalUnit getPermanentOrganizationalUnitWithRole(DLCMTestConstants.OrganizationalUnitStatus organizationalUnitStatus) {
    this.restClientTool.sudoAdmin();
    OrganizationalUnit organizationalUnit = this.getOrCreateOrganizationalUnit(DLCMTestConstants.PersistenceMode.PERMANENT,
            organizationalUnitStatus,
            this.role(), AuthApplicationRole.USER_ID, this.inheritedRole());
    this.restClientTool.exitSudo();
    return organizationalUnit;
  }

  @Override
  protected void deleteFixtures() {
    super.deleteFixtures();
  }
}
