/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Integration Tests - AbstractPreingestIT.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.test.preingest;

import static ch.dlcm.DLCMConstants.PERMANENT_DEPOSIT;
import static org.junit.jupiter.api.Assertions.assertEquals;

import java.util.Collections;
import java.util.List;

import ch.unige.solidify.exception.SolidifyRuntimeException;

import ch.dlcm.model.AbstractDataFile.DataFileStatus;
import ch.dlcm.model.preingest.Deposit;
import ch.dlcm.model.preingest.DepositDataFile;
import ch.dlcm.test.AbstractIT;
import ch.dlcm.test.DLCMTestConstants;

public abstract class AbstractPreingestIT extends AbstractIT {

  @Override
  protected void deleteFixtures() {
  }

  protected boolean isAllReadyInDataFileList(List<DepositDataFile> dataFileList) {
    return dataFileList.stream().allMatch(df -> df.getStatus() == DataFileStatus.READY);
  }

  protected void waitDataFileIsReady(String depositId, int dataFileNumber) {
    List<DepositDataFile> dataFileList = this.depositService.getDepositDataFile(depositId);
    int count = 0;

    // Test if data file is present
    assertEquals(dataFileNumber, dataFileList.size());

    while (!this.isAllReadyInDataFileList(dataFileList)) {
      String initialMessage = "Deposit data files of deposit " + depositId + " are not all in ready status.";
      StringBuilder builder = new StringBuilder().append(initialMessage);
      for (final DepositDataFile dataFile : dataFileList) {
        builder.append("\n Deposit datafile ")
                .append(dataFile.getResId())
                .append(" has status ")
                .append(dataFile.getStatus());
      }
      DLCMTestConstants.failAfterNAttempt(count, builder.toString());
      dataFileList = this.depositService.getDepositDataFile(depositId);
      count++;
    }
  }

  public Deposit getPermanentDeposit() {
    List<Deposit> listDeposit = this.depositService.searchByProperties(Collections.singletonMap("title", PERMANENT_DEPOSIT));
    if (listDeposit.isEmpty()) {
      throw new SolidifyRuntimeException("The permanent deposit is missing.");
    } else if (listDeposit.size() > 1) {
      throw new SolidifyRuntimeException("There should be only one permanent deposit");
    } else {
      return listDeposit.get(0);
    }
  }
}
