/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Integration Tests - CollectionIT.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.test.preingest;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNotNull;
import static org.junit.jupiter.api.Assertions.assertTrue;

import java.util.List;

import org.junit.jupiter.api.Test;

import ch.dlcm.DLCMConstants;
import ch.dlcm.model.oais.ArchivalInfoPackage;
import ch.dlcm.model.preingest.Deposit;
import ch.dlcm.test.DLCMTestConstants;

class CollectionIT extends AbstractPreingestIT {
  private final static int AIC_NUM = 1;
  private final static int AIU_NUM = 3;

  @Override
  protected void setUser() {
    this.restClientTool.sudoAdmin();
  }

  @Test
  void CollectionCreationTest() {

    // Create a deposit object
    final Deposit deposit = this.createRemoteDepositWithContributor(DLCMTestConstants.PersistenceMode.TEMPORARY, DLCMConstants.ARCHIVAL_COLLECTION_LABEL, DEFAULT_DESCRIPTION);

    // Create an AIC with AIU
    final List<ArchivalInfoPackage> aipList = this.createAIC(deposit, AIU_NUM, false);
    assertEquals(AIU_NUM, aipList.size(), "Error in collection list");

    // Approve deposit
    final Deposit deposit1 = this.approveDeposit(deposit);
    assertNotNull(deposit1, "Error deposit approval");

    // Check status
    assertTrue(this.waitDepositIsCompleted(deposit1.getResId()), "Deposit not completed without error");
  }

  @Test
  void CollectionOfCollectionCreationTest() {

    // Create a deposit object
    final Deposit deposit = this.createRemoteDepositWithContributor(DLCMTestConstants.PersistenceMode.TEMPORARY, "Archival Collection of Collection", DEFAULT_DESCRIPTION);

    // Create an AIC with AIC
    final List<ArchivalInfoPackage> aipList = this.createAIC(deposit, AIC_NUM, true);
    assertEquals(AIC_NUM, aipList.size(), "Error in collection list");

    // Approve deposit
    final Deposit deposit1 = this.approveDeposit(deposit);
    assertNotNull(deposit1);

    // Check status
    assertTrue(this.waitDepositIsCompleted(deposit1.getResId()), "Deposit not completed without error");
  }

  @Test
  void HybridCollectionCreationTest() {

    // Create a deposit object
    final Deposit deposit = this.createRemoteDepositWithContributor(DLCMTestConstants.PersistenceMode.TEMPORARY, "Archival Hybrid Collection", DEFAULT_DESCRIPTION);

    // Create an AIC with AIC
    List<ArchivalInfoPackage> aipList = this.createAIC(deposit, AIC_NUM, true);
    assertEquals(AIC_NUM, aipList.size(), "Error in collection list for adding collection");
    aipList = this.createAIC(deposit, AIU_NUM, false);
    assertEquals(AIC_NUM + AIU_NUM, aipList.size(), "Error in collection list for adding units");

    // Approve deposit
    final Deposit deposit1 = this.approveDeposit(deposit);
    assertNotNull(deposit1);

    // Check status
    assertTrue(this.waitDepositIsCompleted(deposit1.getResId()), "Deposit not completed without error");
  }

}
