/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Model - DLCMConstants.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm;

import java.util.Arrays;
import java.util.List;

import ch.unige.solidify.OAIConstants;
import ch.unige.solidify.SolidifyConstants;
import ch.unige.solidify.rest.ActionName;

import ch.dlcm.rest.DLCMActionName;
import ch.dlcm.rest.ModuleName;
import ch.dlcm.rest.ResourceName;
import ch.dlcm.rest.UrlPath;

public class DLCMConstants {

  private DLCMConstants() {
    throw new IllegalStateException(SolidifyConstants.TOOL_CLASS);
  }

  // Modules
  public static final String DLCM = "DLCM";
  public static final String DLCM_SOLUTION = "DLCM Solution";
  public static final String DLCM_WEB_SITE = "https://www.dlcm.ch";
  public static final String DLCM_URI_FINAL_ACCESS_LEVEL = DLCM_WEB_SITE + "/access-level/final";
  public static final String DLCM_URI_EMBARGO_ACCESS_LEVEL = DLCM_WEB_SITE + "/access-level/embargo";
  public static final String DLCM_URI_DATA_TAG = DLCM_WEB_SITE + "/data-tag";

  // URLs
  // @formatter:off
  private static final String[] PUBLIC_URLS = {
          "/docs/**",
          "/DLCM.png",
          "/DLCM.svg",
          "/**/css/*.css",
          "/.well-known/jwks.json",
          "/**/" + OAIConstants.OAI_RESOURCE,
          "/**/" + SolidifyConstants.SCHEMA,
          "/**/" + ResourceName.PROFILE,
          "/**/" + DLCMActionName.LIST_VERSION,
          "/**/" + SolidifyConstants.XSL,
          "/**/" + SolidifyConstants.ORCID + "/" + ActionName.LANDING,
          "/**/" + ModuleName.ADMIN + "/" + ResourceName.SYSTEM_PROPERTY,
          "/**/" + ModuleName.ADMIN + "/" + ResourceName.GLOBAL_BANNER + "/" + DLCMActionName.GET_ACTIVE,
          "/**/" + ModuleName.ADMIN + "/" + ResourceName.LICENSE + "/*/" + DLCMActionName.DOWNLOAD_LOGO,
          "/**/" + ModuleName.ADMIN + "/" + ResourceName.LICENSE + "/*",
          "/**/" + ModuleName.ADMIN + "/" + ResourceName.INSTITUTION + "/*/" + DLCMActionName.DOWNLOAD_LOGO,
          "/**/" + ModuleName.ADMIN + "/" + ResourceName.INSTITUTION + "/*",
          "/**/" + ModuleName.ADMIN + "/" + ResourceName.FUNDING_AGENCY + "/*/" + DLCMActionName.DOWNLOAD_LOGO,
          "/**/" + ModuleName.ADMIN + "/" + ResourceName.FUNDING_AGENCY + "/*",
          "/**/" + ModuleName.ACCESS + "/" + ResourceName.SYSTEM_PROPERTY,
          "/**/" + ModuleName.ACCESS + "/" + ResourceName.PUBLIC_METADATA + "/**",
          "/**/" + ModuleName.ACCESS + "/" + ResourceName.ORG_UNIT + "/**",
          "/**/" + ModuleName.ACCESS + "/" + ResourceName.PUBLIC_METADATA + "/search",
          "/**/" + UrlPath.ACCESS_AIP + "/**/" + ActionName.DOWNLOAD_TOKEN,
          "/**/" + UrlPath.ACCESS_DIP + "/**/" + ActionName.DOWNLOAD_TOKEN,
          "/**/" + UrlPath.ACCESS_ORDER + "/**/" + ActionName.DOWNLOAD_TOKEN,
          "/**/" + ModuleName.PRES_PLANNING + "/" + ResourceName.MODULE,
          "/**/" + ActionName.DOWNLOAD,
  };
  private static final String[] DISABLED_CSRF_URLS = {
          "/**/" + ModuleName.ACCESS + "/" + ResourceName.PUBLIC_METADATA + "/*/prepare-download",
          "/**/" + ModuleName.ACCESS + "/" + ResourceName.PUBLIC_METADATA + "/search",
          "/**/" + OAIConstants.OAI_RESOURCE
  };
  // @formatter:on

  // Default Values
  public static final int DAYS_BY_YEAR = 365;
  public static final int DAYS_BY_MONTH = 30;
  public static final int YEARS_BY_CENTURY = 100;
  public static final String POLICY = "policy";
  public static final String RETENTION = "retention";
  public static final String DISPOSITION_APPROVAL = "disposition approval";
  public static final int ORDER_INCREMENT = 10;

  // Database column name
  public static final String DB_RES_ID = SolidifyConstants.DB_RES_ID; // "resId"
  public static final String DB_ORG_UNIT_ID = "orgUnitId";
  public static final String DB_POLICY_ID = "policyId";
  public static final String DB_INSTITUTION_ID = "institutionId";
  public static final String DB_PERSON_ID = "personId";
  public static final String DB_ROLE_ID = "roleId";
  public static final String DB_AGENCY_ID = "agencyId";
  public static final String DB_PACKAGE_ID = "packageId";
  public static final String DB_SIP_ID = "sipId";
  public static final String DB_AIP_ID = "aipId";
  public static final String DB_DIP_ID = "dipId";
  public static final String DB_COLLECTION_ID = "collectionId";
  public static final String DB_ORDER_ID = "orderId";
  public static final String DB_FILE_ID = "fileId";
  public static final String DB_JOB_ID = "jobId";
  public static final String DB_JOB_EXECUTION_ID = "jobExecutionId";
  public static final String DB_USER_ID = "userId";
  public static final String DB_RATING_TYPE_ID = "ratingTypeId";
  public static final String DB_ARCHIVE_ID = "archiveId";
  public static final String DB_DEPOSIT_ID = "depositId";
  public static final String DB_DOMAIN_ID = "domainId";
  public static final String DB_LANGUAGE_ID = SolidifyConstants.DB_LANGUAGE_ID; // "languageId"
  public static final String DB_RESEARCH_DOMAIN_ID = "researchDomainId";
  public static final String DB_LOGO_ID = "logoId";
  public static final String DB_AVATAR_ID = "avatarId";
  public static final String DB_THUMBNAIL_ID = "thumbnailId";
  public static final String DB_INDEX_FIELD_ALIAS_ID = "indexFieldAliasId";
  public static final String DB_NOTIFICATION_TYPE_ID = "notification_type_id";
  public static final String ORG_UNIT_ID_FIELD = "organizationalUnitId";

  // Index
  public static final String REGISTRY = "registry";
  public static final String PRIVATE = "private";

  // Facets
  public static final String INDEXING_KEYWORD = ".keyword";
  public static final String METADATA_VERSION_FACET = "metadata-versions";
  public static final String ORG_UNIT_FACET = "organizational-units";
  public static final String ACCESS_LEVEL_FACET = "access-levels";
  public static final String DATA_TAG_FACET = "data-tags";
  public static final String CREATOR_FACET = "contributors";
  public static final String FORMAT_FACET = "formats";
  public static final String PATH_FACET = "paths";
  public static final String TYPE_FACET = "types";
  public static final String RETENTION_FACET = "retentions";
  public static final String ARCHIVE_FACET = "archive-types";
  public static final String INSTITUTION_FACET = "institutions";
  public static final String INSTITUTION_DESC_FACET = "institution-descriptions";

  // Schema
  public static final String XSL_HOME = "xslt";

  // DLCM folders
  public static final String METADATA_FOLDER = "metadata";
  public static final String SCHEMA_FOLDER = SolidifyConstants.SCHEMA_HOME;
  public static final String DOC_FOLDER = "documentation";
  public static final String DATA_FOLDER = "data";
  public static final String RESEARCH_DATA_FOLDER = "researchdata";
  public static final String SOFTWARE_FOLDER = "software";
  public static final String PACKAGE = "package";
  public static final String TOMBSTONE = "tombstone";
  public static final String NEW_PACKAGE = "new-package";
  public static final String INTERNAL_FOLDER = "dlcm";
  public static final String UPDATE_FOLDER = "update";

  // DLCM Mime-type
  public static final String DEPOSIT_MIME_TYPE = "application/dlcm-deposit";
  public static final String SIP_MIME_TYPE = "application/dlcm-sip";
  public static final String AIP_MIME_TYPE = "application/dlcm-aip";
  public static final String DIP_MIME_TYPE = "application/dlcm-dip";

  // DLCM AIP
  public static final String AIP = "aip";
  public static final String AIP_PREFIX = "aip-";
  public static final String AIP_CREATION = AIP_PREFIX + "creation";
  public static final String AIP_LAST_UPDATE = AIP_PREFIX + "last-update";
  public static final String AIP_CONTAINER = AIP_PREFIX + "container";
  public static final String AIP_UNIT = AIP_PREFIX + "unit";
  public static final String AIP_TYPE = AIP_PREFIX + "type";
  public static final String AIP_ORGA_UNIT = AIP_PREFIX + "organizational-unit";
  public static final String AIP_RETENTION = AIP_PREFIX + RETENTION;
  public static final String AIP_RETENTION_END = AIP_PREFIX + RETENTION + "-end";
  public static final String AIP_RETENTION_DURATION = AIP_PREFIX + RETENTION + "-duration";
  public static final String AIP_DISPOSITION_APPROVAL = AIP_PREFIX + "disposition-approval";
  public static final String AIP_SIZE = AIP_PREFIX + "size";
  public static final String AIP_ORGA_UNIT_NAME = AIP_PREFIX + "organizational-unit-name";
  public static final String AIP_RESEARCH_DOMAINS = AIP_PREFIX + "research-domains";
  public static final String AIP_FILE_NUMBER = AIP_PREFIX + "file-number";
  public static final String AIP_METADATA_FILE_NUMBER = AIP_PREFIX + "metadata-file-number";
  public static final String AIP_TOTAL_FILE_NUMBER = AIP_PREFIX + "total-file-number";
  public static final String AIP_ACCESS_LEVEL = AIP_PREFIX + "access-level";
  public static final String AIP_EMBARGO_LEVEL = AIP_PREFIX + "embargo-level";
  public static final String AIP_EMBARGO_END_DATE = AIP_PREFIX + "embargo-date";
  public static final String AIP_LICENSE = AIP_PREFIX + "license";
  public static final String AIP_TOMBSTONE_SIZE = AIP_PREFIX + "tombstone-size";
  public static final String AIP_TITLE = AIP_PREFIX + "title";
  public static final String AIP_COLLECTION = AIP_PREFIX + "collection";
  public static final String AIP_THUMBNAIL = AIP_PREFIX + "thumbnail";
  public static final String AIP_DATA_TAG = AIP_PREFIX + "data-tag";
  public static final String AIP_COMPLIANCE_LEVEL = AIP_PREFIX + "compliance-level";
  public static final String AIP_INSTITUTIONS = AIP_PREFIX + INSTITUTION_FACET;
  public static final String AIP_INSTITUTION_DESCRIPTIONS = AIP_PREFIX + INSTITUTION_DESC_FACET;

  // DLCM metadata files
  public static final String METADATA_FILE = "dlcm.xml";
  public static final String REPRESENTATION_INFO_METADATA_FILE = "dlcm-representation-info-";
  public static final String METS_METADATA_FILE = "dlcm-mets.xml";
  public static final String UPDATED_METADATA_FILE = "updated-metadata-";
  public static final String UPDATE_PACKAGE_FILE = "update-package";
  public static final String METS = "METS";

  // DLCM Thumbnails
  public static final String THUMBNAIL_EXTENSION = ".thumbnail";
  public static final String DATASET_THUMBNAIL = "dataset" + THUMBNAIL_EXTENSION;

  // DLCM Metadata
  public static final String FILE_ROOT = "DLCM Files Root";
  public static final String STRUCTURE_MAP = "DLCM Structural Map";
  public static final String METADATA_ROOT = "metadata";
  public static final String DOC_ROOT = "documentation";
  public static final String DATA_ROOT = "researchData";
  public static final String SOFTWARE_ROOT = "software";
  public static final String INTERNAL_ROOT = "dlcm";
  public static final String AGENT_SW = "Software";

  // Search
  public static final String SEARCH_ORG_UNIT = "orgUnit";

  // Metadata field name
  public static final String DATACITE_XML_FIELD = "datacite.xml";
  public static final String DATACITE_FIELD = "datacite";
  public static final String PREMIS_FIELD = "premis";
  public static final String METADATA_VERSION_FIELD = "metadata-version";
  public static final String CREATION_FIELD = "creation";
  public static final String DOI = "DOI";
  public static final String HANDLE = "Handle";
  public static final String ABSTRACT = "Abstract";
  public static final String DATACITE_FAMILY_NAME = "datacite:familyName";
  public static final String DATACITE_GIVEN_NAME = "datacite:givenName";
  public static final String DATACITE_IDENTIFIER_NAME = "datacite:nameIdentifier";
  public static final String DATACITE_RELATIONS_NAME = "datacite:relatedIdentifiers";
  public static final String DATACITE_RELATION_NAME = "datacite:relatedIdentifier";
  public static final String DATACITE_RELATION_ID_TYPE_NAME = "relatedIdentifierType";
  public static final String DATACITE_RELATION_TYPE_NAME = "relationType";
  public static final String DATACITE_SUBJECTS_NAME = "datacite:subjects";
  public static final String DATACITE_RESOURCE_FIELD = "datacite.datacite:resource";
  public static final String IDENTIFIER_FIELD = DATACITE_RESOURCE_FIELD + ".datacite:identifier.content";
  public static final String IDENTIFIER_TYPE_FIELD = DATACITE_RESOURCE_FIELD + ".datacite:identifier.identifierType";
  public static final String RIGHTS_FIELD = DATACITE_RESOURCE_FIELD + ".datacite:rightsList.datacite:rights";
  public static final String DATES_FIELD = DATACITE_RESOURCE_FIELD + ".datacite:dates.datacite:date";
  public static final String CREATORS_FIELD = DATACITE_RESOURCE_FIELD + ".datacite:creators";
  public static final String CREATOR_FIELD = DATACITE_RESOURCE_FIELD + ".datacite:creators.datacite:creator";
  public static final String CREATOR_NAME_FIELD = DATACITE_RESOURCE_FIELD + ".datacite:creators.datacite:creator.datacite:creatorName.content";
  public static final String FORMAT_FIELD = DATACITE_RESOURCE_FIELD + ".datacite:formats.datacite:format";
  public static final String RESOURCE_TYPE_FIELD = DATACITE_RESOURCE_FIELD + ".datacite:resourceType.content";
  public static final String RESOURCE_TYPE_GENERAL_FIELD = DATACITE_RESOURCE_FIELD + ".datacite:resourceType.resourceTypeGeneral";
  public static final String TITLE_FIELD = DATACITE_RESOURCE_FIELD + ".datacite:titles.datacite:title.content";
  public static final String PUBLISHER_FIELD = DATACITE_RESOURCE_FIELD + ".datacite:publisher";
  public static final String DESCRIPTION_TYPE_FIELD = DATACITE_RESOURCE_FIELD + ".datacite:descriptions.datacite:description.descriptionType";
  public static final String DESCRIPTION_CONTENT_FIELD = DATACITE_RESOURCE_FIELD + ".datacite:descriptions.datacite:description.content";
  public static final String DESCRIPTION_LANGUAGE_FIELD = DATACITE_RESOURCE_FIELD + ".datacite:descriptions.datacite:description.xml:lang";
  public static final String RELATIONS_FIELD = DATACITE_RESOURCE_FIELD + "." + DATACITE_RELATIONS_NAME;
  public static final String RELATION_FIELD = RELATIONS_FIELD + "." + DATACITE_RELATION_NAME;
  public static final String SUBJECT_LIST_FIELD = DATACITE_RESOURCE_FIELD + "." + DATACITE_SUBJECTS_NAME;
  public static final String SUBJECT_SCHEME_FIELD = DATACITE_RESOURCE_FIELD + "." + DATACITE_SUBJECTS_NAME + ".datacite:subject.subjectScheme";
  public static final String SUBJECT_CONTENT_FIELD = DATACITE_RESOURCE_FIELD + "." + DATACITE_SUBJECTS_NAME + ".datacite:subject.content";

  public static final String DATACITE_TYPE_BOOK = "Book";
  public static final String DATACITE_TYPE_BOOK_CHAPTER = "BookChapter";
  public static final String DATACITE_TYPE_CONFERENCE_PAPER = "ConferencePaper";
  public static final String DATACITE_TYPE_DATASET = "Dataset";
  public static final String DATACITE_TYPE_JOURNAL_ARTICLE = "JournalArticle";
  public static final String DATACITE_TYPE_PEER_REVIEW = "PeerReview";
  public static final String DATACITE_TYPE_REPORT = "Report";

  public static final String DATE_AVAILABLE_ATTRIBUTE = "Available";
  public static final String DATE_ISSUED_ATTRIBUTE = "Issued";
  public static final String DATE_CREATED_ATTRIBUTE = "Created";
  public static final String DATE_UPDATED_ATTRIBUTE = "Updated";
  public static final String DATE_ACCEPTED_ATTRIBUTE = "Accepted";
  public static final String DATE_COLLECTED_START_ATTRIBUTE = "CollectedStart";
  public static final String DATE_COLLECTED_END_ATTRIBUTE = "CollectedEnd";

  public static final String CUSTOM_METADATA_FIELD = "custom-metadata";
  public static final String ITEMS_FIELD = "items";
  public static final String ITEM_FIELD = "item";
  public static final String ID_FIELD = "id";
  public static final String TYPE_FIELD = "type";
  public static final String ARCHIVE_ID_FIELD = "archiveId";
  public static final String CONTENT_FIELD = "content";
  public static final String KEYWORDS = "keywords";
  public static final String FAMILY_NAME = "familyName";
  public static final String GIVEN_NAME = "givenName";
  public static final String PERSON_ORCID = "orcid";
  public static final String DATE_TYPE = "dateType";

  // BagIt metadata
  public static final String BAGIT_PROFILE = "BagIt-Profile-DLCM.json";
  public static final String BAGIT_DATE = "Bagging-Date";
  public static final String BAGIT_SOURCE = "Source-Organization";
  public static final String BAGIT_SOURCE_ADDR = "Organization-Address";
  public static final String BAGIT_ARC_ID = "DLCM-Id";
  public static final String BAGIT_METADATA_SCHEMA = "DLCM-Metadata-Schema";
  public static final String BAGIT_ARC_NAME = "DLCM-Title";
  public static final String BAGIT_ARC_RETENTION = "DLCM-Retention";
  public static final String BAGIT_ARC_RETENTION_DURATION = "DLCM-Retention-Duration";
  public static final String BAGIT_ARC_DISPOSITION_APPROVAL = "DLCM-Disposition-Approval";
  public static final String BAGIT_ORG_UNIT_ID = "DLCM-OrgUnit-Id";
  public static final String BAGIT_ORG_UNIT_NAME = "DLCM-OrgUnit-Name";
  public static final String BAGIT_PACKAGE_SIZE = "DLCM-Package-Size";
  public static final String BAGIT_PACKAGE_FILE_NUMBER = "DLCM-Package-File-Number";
  public static final String FORMAT_XML = "text/xml";

  // OAI
  public static final String OAI_ORG_UNIT = "orgunit";
  public static final String OAI_DATACITE = "datacite";
  public static final String OAI_SCHEMA_ORG = "schema.org";
  public static final String OAI_SCHEMA_ORG_NAME = "schema.org Dataset Metadata";
  public static final String OAI_SCHEMA_ORG_SCHEMA = "https://schema.org/Dataset";

  // DataCite
  public static final String DATACITE_NAME = "DataCite Metadata";
  public static final String DATACITE_SCHEMA_4_3 = "https://schema.datacite.org/meta/kernel-4.3/metadata.xsd";

  // XML Namespace
  public static final String DATACITE_NAMESPACE_4 = "http://datacite.org/schema/kernel-4";
  public static final String PREMIS_NAMESPACE_3 = "http://www.loc.gov/premis/v3";
  public static final String METS_NAMESPACE = "http://www.loc.gov/METS/";
  public static final String DLCM_SCHEMA_1 = "dlcm-1.0.xsd";
  public static final String DLCM_NAMESPACE_1 = "http://www.dlcm.ch/dlcm/v1";
  public static final String DLCM_SCHEMA_2 = "dlcm-2.0.xsd";
  public static final String DLCM_NAMESPACE_2 = "http://www.dlcm.ch/dlcm/v2";

  // Preservation Job
  public static final String REPORT_ID = "{reportId}";

  // Misc.
  public static final String UNKNOWN = "unknown";
  public static final String ORCID = "ORCID";
  public static final String ROR = "ROR";
  public static final String WEB = "WEB";
  public static final String GRID = "GRID";
  public static final String ISNI = "ISNI";
  public static final String CROSSREF_FUNDER = "CrossrefFunder";
  public static final String WIKIDATA = "Wikidata";
  public static final String SPDX = "SPDX";
  public static final String NO_REPLY_PREFIX = "noreply-";
  public static final String REASON = "reason";
  public static final String GRADE = "grade";
  public static final String RATING_TYPE = "ratingType";

  // Tests
  public static final String PERMANENT_TEST_DATA_LABEL = "[Permanent Test Data] ";
  public static final String TEMPORARY_TEST_DATA_LABEL = "[Temporary Test Data] ";
  public static final String ARCHIVAL_UNIT_LABEL = "Archival Unit";
  public static final String ARCHIVAL_COLLECTION_LABEL = "Archival Collection";
  public static final String PERMANENT_DEPOSIT = "Permanent Deposit";
  public static final String TEST_RES_ID = "test";

  // Message processor
  public static final String MESSAGE_PROCESSOR_NAME = "DLCM Message Processor";
  public static final String BAD_SEARCH_REQUEST = "Bad search request";

  // XML
  public static final String XML_NAMESPACE_PREFIX_MAPPER = "com.sun.xml.bind.namespacePrefixMapper";

  // JPA keys
  public static final String INFO_PACKAGE_RES_ID_KEY = "infoPackage.resId";

  // Emails
  public static final String NOTIFICATION_EMAIL_DLCM_PREFIX = "[DLCM]";
  public static final String NOTIFICATION_EMAIL_VALIDATION_PREFIX = "[s validation]";
  public static final String NOTIFICATION_IDS = "notificationIds";

  // Checksums
  public static final String FILE = "file";
  public static final String CATEGORY = "category";
  public static final String TYPE = "type";
  public static final String FOLDER = "folder";
  public static final String METADATA_TYPE = "metadataType";
  public static final String CHECKSUM_MD5 = "checksumMd5";
  public static final String CHECKSUM_MD5_ORIGIN = "checksumMd5Origin";
  public static final String CHECKSUM_SHA1 = "checksumSha1";
  public static final String CHECKSUM_SHA1_ORIGIN = "checksumSha1Origin";
  public static final String CHECKSUM_SHA256 = "checksumSha256";
  public static final String CHECKSUM_SHA256_ORIGIN = "checksumSha256Origin";
  public static final String CHECKSUM_CRC32 = "checksumCrc32";
  public static final String CHECKSUM_CRC32_ORIGIN = "checksumCrc32Origin";

  // Web service Parameter
  public static final String ARCHIVAL_UNIT_PARAM = "archivalUnit";
  public static final String ARCHIVE_MASTER_TYPE_ID_PARAM = "masterType.resId";
  public static final String IDENTIFIER_TYPE_PARAM = "identifierType";

  // Licenses
  public static final String OPEN_LICENSE_ID_FIELD = "openLicenseId";
  public static final String DOMAIN_CONTENT_FIELD = "domainContent";
  public static final String DOMAIN_DATA_FIELD = "domainData";
  public static final String DOMAIN_SOFTWARE_FIELD = "domainSoftware";
  public static final String IS_GENERIC_FIELD = "isGeneric";
  public static final String OD_CONFORMANCE_FIELD = "odConformance";
  public static final String OSD_CONFORMANCE_FIELD = "osdConformance";

  private static final List<String> LICENSE_DLCM_CHECK_URI_LIST = Arrays
          .asList(DLCMConstants.DLCM_URI_DATA_TAG, DLCMConstants.DLCM_URI_FINAL_ACCESS_LEVEL, DLCMConstants.DLCM_URI_EMBARGO_ACCESS_LEVEL);

  public static String[] getPublicUrls() {
    return PUBLIC_URLS.clone();
  }

  public static String[] getDisabledCsrfUrls() {
    return DISABLED_CSRF_URLS.clone();
  }

  public static List<String> getUriListWithoutLicense() {
    return LICENSE_DLCM_CHECK_URI_LIST;
  }

}
