/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Model - RegistryMessage.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.message;

import java.util.HashMap;
import java.util.Map;

import ch.unige.solidify.exception.SolidifyRuntimeException;
import ch.unige.solidify.model.ChangeInfo;

public class RegistryMessage extends DlcmMessage {
  private static final long serialVersionUID = 3564185652901279095L;

  private ChangeInfo creation;
  private ChangeInfo lastUpdate;
  private ChangeInfo disposition;

  private Map<String, Object> metadata = new HashMap<>();

  private String type;

  public RegistryMessage(String resId, String type) {
    super(resId);
    if(resId == null) {
      throw new SolidifyRuntimeException("Cannot build a RegistryMessage with a null resID");
    }
    this.setType(type);
  }

  public ChangeInfo getCreation() {
    return this.creation;
  }

  public ChangeInfo getDisposition() {
    return this.disposition;
  }

  public ChangeInfo getLastUpdate() {
    return this.lastUpdate;
  }

  public Map<String, Object> getMetadata() {
    return this.metadata;
  }

  public String getType() {
    return this.type;
  }

  public void setCreation(ChangeInfo creation) {
    this.creation = creation;
  }

  public void setDisposition(ChangeInfo disposition) {
    this.disposition = disposition;
  }

  public void setLastUpdate(ChangeInfo lastUpdate) {
    this.lastUpdate = lastUpdate;
  }

  public void setMetadata(Map<String, Object> metadata) {
    this.metadata = metadata;
  }

  public void setType(String type) {
    this.type = type;
  }

  @Override
  public String toString() {
    return "RegistryMessage [resId=" + this.getResId() + ", type=" + this.getType() + "]";
  }
}
