/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Model - DataCategory.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.model;

import java.util.ArrayList;
import java.util.List;

import ch.unige.solidify.rest.ActionName;

@SuppressWarnings("squid:S00115")
public enum DataCategory {

  // @formatter:off
  Primary(null),
    Observational(DataCategory.Primary),
    Experimental(DataCategory.Primary),
    Simulation(DataCategory.Primary),
    Derived(DataCategory.Primary),
    Reference(DataCategory.Primary),
    Digitalized(DataCategory.Primary),
  Secondary(null),
    Publication(DataCategory.Secondary),
    DataPaper(DataCategory.Secondary),
    Documentation(DataCategory.Secondary),
  Package(null),
    InformationPackage(DataCategory.Package),
    UpdatePackage(DataCategory.Package),
    Metadata(DataCategory.Package),
    CustomMetadata(DataCategory.Package),
    UpdatedMetadata(DataCategory.Package),
  Software(null),
    Code(DataCategory.Software),
    Binaries(DataCategory.Software),
    VirtualMachine(DataCategory.Software),
  Internal(null),
    DatasetThumbnail(DataCategory.Internal),
    DatafileThumbnail(DataCategory.Internal);
  // @formatter:on

  public static boolean check(DataCategory cat, DataCategory subcat) {
    return listCategories().contains(cat) && listCategories(cat).contains(subcat);
  }

  public static List<DataCategory> listCategories() {
    return listCategories(null);
  }

  public static List<DataCategory> listCategories(DataCategory parent) {
    final List<DataCategory> list = new ArrayList<>();
    for (final DataCategory dc : DataCategory.values()) {
      if (dc.category == parent) {
        list.add(dc);
      }
    }
    return list;
  }

  public static List<DataCategory> listSubCategories() {
    final List<DataCategory> list = new ArrayList<>();
    for (final DataCategory dc : DataCategory.values()) {
      if (dc.category != null) {
        list.add(dc);
      }
    }
    return list;

  }

  public static List<DataCategory> listSubCategories(String category) {
    if (category == null || category.isEmpty() || category.equals(ActionName.ALL)) {
      return listSubCategories();
    }
    return listCategories(DataCategory.valueOf(category));
  }

  private DataCategory category = null;

  DataCategory(DataCategory parent) {
    this.category = parent;
  }
}
