/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Model - DataTag.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.model;

public enum DataTag {

  // @formatter:off
  UNDEFINED(0, "Undefined",Access.PUBLIC),
  BLUE(10, "Public", Access.PUBLIC),
  GREEN(20, "Controlled Public", Access.RESTRICTED),
  YELLOW(30, "Accountable", Access.CLOSED),
  ORANGE(40, "More Accountable", Access.CLOSED),
  RED(50, "Fully Accountable", Access.CLOSED),
  CRIMSON(60, "Maximally Restricted", Access.CLOSED);
  // @formatter:on

  private final int level;
  private final String description;
  private final Access minimunAccessLevel;

  DataTag(int level, String description, Access accessLevel) {
    this.level = level;
    this.description = description;
    this.minimunAccessLevel = accessLevel;
  }

  public int value() {
    return this.level;
  }

  public String getDescription() {
    return this.description;
  }

  public Access getMinimunAccessLevel() {
    return this.minimunAccessLevel;
  }

  public boolean isAccessLevelAllowed(Access accessLevelToCheck) {
    switch (this.minimunAccessLevel) {
      case CLOSED:
        return accessLevelToCheck.equals(Access.CLOSED);
      case RESTRICTED:
        return (accessLevelToCheck.equals(Access.RESTRICTED) || accessLevelToCheck.equals(Access.CLOSED));
      case PUBLIC:
        return true;
    }
    return false;
  }

}
