/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Model - PackageStatus.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.model;

public enum PackageStatus {
  CHECKED, CHECKING, CLEANED, CLEANING, COMPLETED, DISPOSABLE, DISPOSAL_APPROVED_BY_ORGUNIT, DISPOSAL_APPROVED, DISPOSED, DOWNLOADING, IN_ERROR, EDITING_METADATA, IN_PREPARATION, IN_PROGRESS, INDEXING, PRESERVATION_ERROR, READY, REINDEXING, RELOADED, REPLICATING_TOMBSTONE, REPLICATING_PACKAGE, RESUBMITTING, STORED, UPDATING_RETENTION;

  public static boolean isDisposalProcess(PackageStatus status) {
    return (status.equals(DISPOSABLE)
            || status.equals(DISPOSAL_APPROVED)
            || status.equals(DISPOSAL_APPROVED_BY_ORGUNIT)
            || status.equals(DISPOSED));
  }

  public static boolean isCleaningProcess(PackageStatus status) {
    return (status.equals(CLEANING)
            || status.equals(CLEANED));
  }

  public static boolean isReindexingProcess(PackageStatus status) {
    return (status.equals(REINDEXING));
  }

  public static boolean isMetadataEditionProcess(PackageStatus status) {
    return (status.equals(EDITING_METADATA));
  }

  public static boolean isCompletedProcess(PackageStatus status) {
    return (status == PackageStatus.COMPLETED
            || status == PackageStatus.CLEANED
            || status == PackageStatus.DISPOSED);
  }

  public static boolean isInError(PackageStatus status) {
    return (status == PackageStatus.IN_ERROR
            || status == PackageStatus.PRESERVATION_ERROR);
  }

  public static boolean isInProgress(PackageStatus status) {
    return (!isInError(status) && !isCompletedProcess(status));
  }

}
