/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Model - ArchiveThumbnail.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.model.index;

import javax.persistence.CascadeType;
import javax.persistence.Entity;
import javax.persistence.JoinColumn;
import javax.persistence.OneToOne;

import com.fasterxml.jackson.annotation.JsonIgnore;

import ch.unige.solidify.model.Image;
import ch.unige.solidify.model.ImageInterface;
import ch.unige.solidify.rest.ResourceNormalized;

import ch.dlcm.DLCMConstants;
import ch.dlcm.rest.ModuleName;

@Entity
public class ArchiveThumbnail extends ResourceNormalized implements ImageInterface {

  @OneToOne(cascade = CascadeType.ALL, orphanRemoval = true)
  @JoinColumn(name = DLCMConstants.DB_THUMBNAIL_ID, referencedColumnName = DLCMConstants.DB_RES_ID)
  private ArchiveThumbnailLogo datasetThumbnail;

  public ArchiveThumbnailLogo getDatasetThumbnail() {
    return this.datasetThumbnail;
  }

  public void setDatasetThumbnail(ArchiveThumbnailLogo datasetThumbnail) {
    this.datasetThumbnail = datasetThumbnail;
  }

  @Override
  public void init() {
    // Do nothing
  }

  @Override
  public String managedBy() {
    return ModuleName.DATAMGMT;
  }

  @Override
  @JsonIgnore
  public Image getNewIfNeededImage() {
    if (this.getDatasetThumbnail() == null) {
      this.setDatasetThumbnail(new ArchiveThumbnailLogo());
    }
    return this.getDatasetThumbnail();
  }

  @Override
  @JsonIgnore
  public Image getImage() {
    return this.getDatasetThumbnail();
  }

  @Override
  public void setImage(Image logo) {
    this.setDatasetThumbnail((ArchiveThumbnailLogo) logo);
  }

}
