/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Model - SubmissionPolicy.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.model.policies;

import java.util.ArrayList;
import java.util.List;
import javax.persistence.CascadeType;
import javax.persistence.Entity;
import javax.persistence.OneToMany;
import javax.persistence.PrePersist;
import javax.persistence.PreUpdate;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonProperty.Access;

import ch.unige.solidify.exception.SolidifyUndeletableException;
import ch.unige.solidify.rest.ResourceNormalized;

import ch.dlcm.DLCMConstants;
import ch.dlcm.rest.ModuleName;

@Entity
public class SubmissionPolicy extends ResourceNormalized {

  @JsonIgnore
  @OneToMany(mappedBy = OrganizationalUnitSubmissionPolicy.PATH_TO_SUBMISSION_POLICY, cascade = CascadeType.ALL, orphanRemoval = true)
  protected List<OrganizationalUnitSubmissionPolicy> organizationalUnits = new ArrayList<>();

  @NotNull
  @Size(min = 1)
  private String name;

  private Boolean submissionApproval;

  // Default => 2 months, measured in days
  private Integer timeToKeep;

  @Override
  public boolean equals(Object obj) {
    if (this == obj) {
      return true;
    }
    if (!super.equals(obj)) {
      return false;
    }
    if (this.getClass() != obj.getClass()) {
      return false;
    }
    final SubmissionPolicy other = (SubmissionPolicy) obj;
    return this.submissionApproval == other.submissionApproval;
  }

  public String getName() {
    return this.name;
  }

  public Boolean getSubmissionApproval() {
    return this.submissionApproval;
  }

  public Integer getTimeToKeep() {
    return this.timeToKeep;
  }

  @JsonProperty(access = Access.READ_ONLY)
  public int getUseNumber() {
    return this.organizationalUnits.size();
  }

  @Override
  public int hashCode() {
    final int prime = 31;
    int result = super.hashCode();
    result = prime * result + (this.submissionApproval ? 1231 : 1237);
    return result;
  }

  @PrePersist
  @PreUpdate
  @Override
  public void init() {
    if (this.submissionApproval == null) {
      this.submissionApproval = true;
    }
    if (this.timeToKeep == null) {
      this.timeToKeep = 2 * DLCMConstants.DAYS_BY_MONTH;
    }
  }

  @Override
  public boolean isDeletable() {
    final int nbOrgUnits = this.getUseNumber();
    if (nbOrgUnits > 0) {
      throw new SolidifyUndeletableException("The submission policy " + this.getResId() + " is used in " + nbOrgUnits + " organizational units");
    }
    return true;
  }

  @Override
  public String managedBy() {
    return ModuleName.ADMIN;
  }

  public void setName(String name) {
    this.name = name;
  }

  public void setSubmissionApproval(Boolean submissionApproval) {
    this.submissionApproval = submissionApproval;
  }

  public void setTimeToKeep(Integer timeToKeep) {
    this.timeToKeep = timeToKeep;
  }

  @Override
  public String toString() {
    return super.toString() + " (name=" + this.getName() + ")";
  }
}
