/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Model - JobType.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.model.preservation;

import java.util.ArrayList;
import java.util.List;

public enum JobType {
  // @formatter:off
  ARCHIVE_CHECK("Archive Check", true, true),
  CLEAN_SUBMISSION("Clean Submission", true, false),
  DISPOSAL("Trigger Disposal Processes", true, true),
  FIXITY("Fixity", true, true),
  MIGRATION("Migration", false, false),
  ARCHIVE_PRELOAD_SMALL("Archive [small size] Preloading", true, true),
  ARCHIVE_PRELOAD_BIG("Archive [big size] Preloading", true, true),
  PURGE_SUBMISSION_TEMP_FILES("Purge Submission Temporary Files", true, false),
  PURGE_ORDER("Purge order", true, false),
  REBUILD_REGISTRY("Rebuild Registry", false, false),
  REINDEX("Re-index", false, true),
  RELOAD("Reload", false, false),
  REPLICATION("Replication", true, true),
  REPLICATION_CHECK("Replication Check", true, true),
  CHECK_COMPLIANCE_LEVEL("Check compliance level", true, true);
  // @formatter:on

  public static List<String> getJobType() {
    final List<String> list = new ArrayList<>();
    for (final JobType jt : JobType.values()) {
      list.add(jt.getLabel());
    }
    return list;
  }

  private boolean hasReport;
  private boolean isRecurring;

  private String label;

  JobType(String label, boolean isRecurring, boolean hasReport) {
    this.label = label;
    this.isRecurring = isRecurring;
    this.hasReport = hasReport;
  }

  public boolean getHasReport() {
    return this.hasReport;
  }

  public String getLabel() {
    return this.label;
  }

  public boolean isRecurring() {
    return this.isRecurring;
  }
}
