/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Model - ArchiveUserRating.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.model.settings;

import static ch.dlcm.DLCMRestFields.ARCHIVE_ID_FIELD;
import static ch.dlcm.DLCMRestFields.RATING_TYPE_FIELD;
import static ch.dlcm.DLCMRestFields.RES_ID_FIELD;
import static ch.dlcm.DLCMRestFields.USER_FIELD;
import static ch.unige.solidify.SolidifyConstants.DB_ID_LENGTH;

import java.util.Arrays;
import java.util.List;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.Table;
import javax.persistence.UniqueConstraint;
import javax.validation.constraints.NotNull;

import ch.unige.solidify.rest.ResourceNormalized;

import ch.dlcm.DLCMConstants;
import ch.dlcm.model.security.User;
import ch.dlcm.rest.ModuleName;

@Entity
@Table(name = "archiveUserRatings", uniqueConstraints = {
        @UniqueConstraint(columnNames = { DLCMConstants.DB_ARCHIVE_ID, DLCMConstants.DB_USER_ID, DLCMConstants.DB_RATING_TYPE_ID }) })
public class ArchiveUserRating extends ResourceNormalized {

  @NotNull
  @Column(name = DLCMConstants.DB_ARCHIVE_ID, length = DB_ID_LENGTH)
  private String archiveId;

  @NotNull
  @ManyToOne(targetEntity = User.class)
  @JoinColumn(name = DLCMConstants.DB_USER_ID, referencedColumnName = DLCMConstants.DB_RES_ID)
  private User user;

  @NotNull
  @ManyToOne(targetEntity = RatingType.class)
  @JoinColumn(name = DLCMConstants.DB_RATING_TYPE_ID, referencedColumnName = DLCMConstants.DB_RES_ID)
  private RatingType ratingType;

  @NotNull
  private Integer grade;

  public ArchiveUserRating() {

  }

  public String getArchiveId() {
    return this.archiveId;
  }

  public User getUser() {
    return this.user;
  }

  public RatingType getRatingType() {
    return this.ratingType;
  }

  public void setArchiveId(String archiveId) {
    this.archiveId = archiveId;
  }

  public void setUser(User user) {
    this.user = user;
  }

  public void setRatingType(RatingType ratingType) {
    this.ratingType = ratingType;
  }

  public Integer getGrade() {
    return this.grade;
  }

  public void setGrade(Integer grade) {
    this.grade = grade;
  }

  @Override
  public String managedBy() {
    return ModuleName.ADMIN;
  }

  @Override
  public void init() {

  }

  @Override
  public List<String> getNonUpdatableFields() {
    return Arrays.asList(RES_ID_FIELD, ARCHIVE_ID_FIELD, RATING_TYPE_FIELD, USER_FIELD);
  }

}
