/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Model - ResearchDomain.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.model.settings;

import java.util.ArrayList;
import java.util.List;
import javax.persistence.CollectionTable;
import javax.persistence.Column;
import javax.persistence.ElementCollection;
import javax.persistence.Entity;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToMany;
import javax.persistence.Table;
import javax.persistence.UniqueConstraint;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;

import org.springframework.data.jpa.domain.Specification;
import org.springframework.hateoas.server.mvc.WebMvcLinkBuilder;

import com.fasterxml.jackson.annotation.JsonIgnore;

import ch.unige.solidify.SolidifyConstants;
import ch.unige.solidify.model.Label;
import ch.unige.solidify.rest.SearchableResourceNormalized;
import ch.unige.solidify.util.SearchCriteria;

import ch.dlcm.DLCMConstants;
import ch.dlcm.rest.ModuleName;
import ch.dlcm.specification.ResearchDomainSearchSpecification;

@Entity
@Table(uniqueConstraints = { @UniqueConstraint(columnNames = { "source", "code" }) })
public class ResearchDomain extends SearchableResourceNormalized<ResearchDomain> {

  @NotNull
  @Size(min = 1, max = SolidifyConstants.DB_ID_LENGTH)
  private String code;

  @ElementCollection
  @Column(name = DLCMConstants.DB_DOMAIN_ID)
  @CollectionTable(joinColumns = {
          @JoinColumn(name = DLCMConstants.DB_DOMAIN_ID) }, uniqueConstraints = @UniqueConstraint(columnNames = { DLCMConstants.DB_DOMAIN_ID,
                  DLCMConstants.DB_LANGUAGE_ID }))
  private List<Label> labels = new ArrayList<>();

  @NotNull
  @Size(max = SolidifyConstants.DB_LONG_STRING_LENGTH)
  private String name;

  @NotNull
  @Size(min = 1, max = SolidifyConstants.DB_SHORT_STRING_LENGTH)
  private String source;

  @ManyToMany(mappedBy = "researchDomains")
  @JsonIgnore
  private List<OrganizationalUnit> organizationalUnits;

  @Override
  public void addLinks(WebMvcLinkBuilder linkBuilder, boolean mainRes, boolean subResOnly) {
    super.addLinks(linkBuilder, mainRes, subResOnly);
  }

  public String getCode() {
    return this.code;
  }

  public List<Label> getLabels() {
    return this.labels;
  }

  public String getName() {
    return this.name;
  }

  public String getSource() {
    return this.source;
  }

  public List<OrganizationalUnit> getOrganizationalUnits() {
    return this.organizationalUnits;
  }

  public boolean addOrganizationalUnit(OrganizationalUnit organizationalUnit) {
    boolean result = this.organizationalUnits.add(organizationalUnit);
    if (result && !organizationalUnit.getResearchDomains().contains(this)) {
      result = organizationalUnit.addResearchDomain(this);
    }
    return result;
  }

  public boolean removeOrganizationalUnit(OrganizationalUnit organizationalUnit) {
    boolean result = true;
    if (this.organizationalUnits.contains(organizationalUnit)) {
      result = this.organizationalUnits.remove(organizationalUnit);
      if (result && organizationalUnit.getResearchDomains().contains(this)) {
        result = organizationalUnit.removeResearchDomain(this);
      }
    }
    return result;
  }

  @Override
  public void init() {
    // Do nothing
  }

  @Override
  public String managedBy() {
    return ModuleName.ADMIN;
  }

  public void setCode(String code) {
    this.code = code;
  }

  public void setLabels(List<Label> labels) {
    this.labels = labels;
  }

  public void setName(String name) {
    this.name = name;
  }

  public void setSource(String source) {
    this.source = source;
  }

  @Override
  public Specification<ResearchDomain> getSearchSpecification(SearchCriteria criteria) {
    return new ResearchDomainSearchSpecification(criteria);
  }
}
