/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Model - OrganizationalUnitPersonRoleSpecification.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.specification;

import java.util.List;

import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Path;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;

import ch.unige.solidify.specification.Join3TiersSpecification;

import ch.dlcm.model.settings.OrganizationalUnitPersonRole;
import ch.dlcm.model.settings.Person;

public class OrganizationalUnitPersonRoleSpecification extends Join3TiersSpecification<OrganizationalUnitPersonRole> {
  private static final long serialVersionUID = 1L;

  public OrganizationalUnitPersonRoleSpecification(OrganizationalUnitPersonRole joinCriteria) {
    super(joinCriteria, OrganizationalUnitPersonRole.PATH_TO_ORG_UNIT, OrganizationalUnitPersonRole.PATH_TO_PERSON,
            OrganizationalUnitPersonRole.PATH_TO_ROLE);
  }

  @Override
  protected String getParentId() {
    return this.joinCriteria.getOrganizationalUnit().getResId();
  }

  @Override
  protected String getChildId() {
    return this.joinCriteria.getPerson().getResId();
  }

  @Override
  protected String getGrandChildId() {
    return this.joinCriteria.getRole().getResId();
  }

  @Override
  protected void completeJoinPredicatesList(Root<OrganizationalUnitPersonRole> root, CriteriaQuery<?> query, CriteriaBuilder builder,
          List<Predicate> predicatesList) {
    // do nothing, validation rights have no property in join table
  }

  /**
   * Filter on structure
   *
   * @param root
   * @param query
   * @param builder
   * @param predicatesList
   */
  @Override
  protected void completeChildPredicatesList(Root<OrganizationalUnitPersonRole> root, CriteriaQuery<?> query, CriteriaBuilder builder,
          List<Predicate> predicatesList) {

    Person person = this.joinCriteria.getPerson();
    Path<?> personPath = this.getChildPath(root);

    if (person.getFirstName() != null) {
      predicatesList.add(builder.like(personPath.get("firstName"), "%" + person.getFirstName() + "%"));
    }
    if (person.getLastName() != null) {
      predicatesList.add(builder.like(personPath.get("lastName"), "%" + person.getLastName() + "%"));
    }
    if (person.getOrcid() != null) {
      predicatesList.add(builder.like(personPath.get("orcid"), "%" + person.getOrcid() + "%"));
    }
  }

}
