/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM PreIngest - ZipPreparationServiceTest.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.test.preparation;

import java.nio.file.Paths;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.context.junit.jupiter.SpringExtension;

import ch.unige.solidify.config.SolidifyProperties;

import ch.dlcm.DLCMMetadataVersion;
import ch.dlcm.config.DLCMProperties;
import ch.dlcm.config.DLCMRepositoryDescription;
import ch.dlcm.preparation.zip.ZipPreparationService;
import ch.dlcm.service.IdentifierService;
import ch.dlcm.service.MetadataService;

@ActiveProfiles({ "pre-zip", "sec-noauth" })
@ExtendWith(SpringExtension.class)
class ZipPreparationServiceTest extends PreparationServiceTest {

  @Test
  void preparationWithMetadataTest() {
    for (final DLCMMetadataVersion version : DLCMMetadataVersion.values()) {
      try {
        this.runTestWithMetadata(Paths.get("src", "test", "resources", "deposits").toAbsolutePath(), version);
      } catch (final Exception e) {
        this.returnException(version, e);
      }
    }
  }

  @Test
  void preparationWithoutMetadataTest() {
    for (final DLCMMetadataVersion version : DLCMMetadataVersion.values()) {
      try {
        this.runTestWithoutMetadata(Paths.get("src", "test", "resources", "deposits").toAbsolutePath(), version);
      } catch (final Exception e) {
        this.returnException(version, e);
      }
    }
  }

  @BeforeEach
  void setup() {
    super.setUp();
    // GitInfo
    // DLCM Properties
    this.config = new DLCMProperties(new SolidifyProperties());
    this.config.setHome(this.workingDir.toAbsolutePath().toString());
    this.config.getParameters().setDefaultChecksum("MD5");
    this.config.getParameters().setChecksumList(new String[] { "MD5" });
    // DLCM Repository description
    this.repositoryDescription = new DLCMRepositoryDescription(this.gitInfoProperties);
    // Services
    this.preparationService = new ZipPreparationService(
            this.config,
            this.messageService,
            this.historyService,
            this.depositService,
            this.depositDataFileService,
            this.fallbackSipRemoteService,
            this.fallbackAipRemoteService,
            this.fallbackArchiveMetadataRemoteService);
    this.metadataService = new MetadataService(
            this.fallbackLanguageRemoteService,
            this.fallbackOrgUnitRemoteService,
            this.fallbackAipRemoteService,
            this.historyService,
            this.config,
            this.repositoryDescription,
            this.messageService,
            this.gitInfoProperties,
            this.fallbackMetadataTypeRemoteService,
            this.fallbackPersonRemoteService,
            this.fallbackLicenseRemoteService,
            this.fallbackArchiveMetadataRemoteService,
            this.fallbackPreservationPolicyRemoteService,
            this.fallbackArchiveTypeResourceService);
    this.identifierService = new IdentifierService(this.repositoryDescription);
  }

}
