/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Resource Server Common - AbstractDataFileController.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.controller;

import ch.unige.solidify.exception.SolidifyResourceNotFoundException;
import ch.unige.solidify.rest.Resource;
import ch.unige.solidify.rest.Result;
import ch.unige.solidify.rest.SubResourceContainer;

import ch.dlcm.model.AbstractDataFile;
import ch.dlcm.model.AbstractDataFile.DataFileStatus;
import ch.dlcm.model.oais.MetadataVersionAware;

public abstract class AbstractDataFileController<T extends Resource & SubResourceContainer & MetadataVersionAware, V extends Resource>
        extends DLCMCompositionController<T, V> {
  protected <D extends AbstractDataFile> Result resumeDataFile(String parentid, String id) {
    final V item = this.checkSubresourceExistsAndIsLinkedToParent(id, parentid);
    final D itemDataFile = (D) item;
    if (item == null) {
      throw new SolidifyResourceNotFoundException(parentid + "-" + id);
    }
    final Result res = new Result(item.getResId());
    if (itemDataFile.getStatus() == DataFileStatus.READY
            || itemDataFile.getStatus() == DataFileStatus.CLEANED) {
      res.setMesssage(this.messageService.get("message.datafile.completed", new Object[] { id, parentid }));
    } else if (itemDataFile.getStatus() == DataFileStatus.TO_PROCESS
            || itemDataFile.getStatus() == DataFileStatus.CLEANING) {
      res.setMesssage(this.messageService.get("message.datafile.inprogress", new Object[] { id, parentid }));
    } else {
      res.setStatus(Result.ActionStatus.NOT_EXECUTED);
      itemDataFile.setFileStatus(DataFileStatus.RECEIVED);

      this.subResourceService.save(item);
      res.setStatus(Result.ActionStatus.EXECUTED);
      res.setMesssage(this.messageService.get("message.datafile.resume", new Object[] { id, parentid }));
    }
    return res;
  }
}
