/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Resource Server Common - DownloadTokenService.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.service;

import java.time.OffsetDateTime;

import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;

import ch.dlcm.config.DLCMProperties;
import ch.dlcm.model.settings.DownloadToken;
import ch.dlcm.repository.DownloadTokenRepository;

@Service
public class DownloadTokenService {

  private final int downloadTokenLifeTimeMinutes;
  private final DownloadTokenRepository downloadTokenRepository;

  private static final long CLEANING_INTERVAL_MILLISECONDS = 600000;  // ten minutes

  public DownloadTokenService(DownloadTokenRepository downloadTokenRepository, DLCMProperties config) {
    this.downloadTokenRepository = downloadTokenRepository;
    this.downloadTokenLifeTimeMinutes = config.getSecurity().getDownloadTokenLifeTimeMinutes();
  }

  @Scheduled(fixedDelay = CLEANING_INTERVAL_MILLISECONDS)
  public void removeExpiredDownloadToken() {
    final OffsetDateTime limitTime = OffsetDateTime.now().minusMinutes(this.downloadTokenLifeTimeMinutes);
    for (DownloadToken downloadToken : this.downloadTokenRepository.findExpiredDownloadTokens(limitTime)) {
      this.downloadTokenRepository.delete(downloadToken);
    }
  }

}
