/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Resource Server Common - ArchiveDownloadRemoteResourceService.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.service.rest.abstractservice;

import ch.unige.solidify.service.SolidifyRestClientService;

import ch.dlcm.config.DLCMProperties;
import ch.dlcm.rest.DLCMActionName;
import ch.dlcm.rest.ResourceName;

public abstract class ArchiveDownloadRemoteResourceService {

  protected final DLCMProperties config;
  protected final SolidifyRestClientService restClientService;

  public ArchiveDownloadRemoteResourceService(DLCMProperties config, SolidifyRestClientService restClientService) {
    this.config = config;
    this.restClientService = restClientService;
  }

  public String getArchiveDownloadStatus(String resId) {
    this.restClientService.checkResId(resId);
    String url = this.config.getModule().getAccess().getUrl() + "/" + ResourceName.PUBLIC_METADATA + "/" + resId + "/" +
            DLCMActionName.DOWNLOAD_STATUS;
    return this.restClientService.getResource(url, String.class);
  }

  public String prepareArchiveDownload(String resId) {
    this.restClientService.checkResId(resId);
    String url = this.config.getModule().getAccess().getUrl() + "/" + ResourceName.PUBLIC_METADATA + "/" + resId + "/" +
            DLCMActionName.PREPARE_DOWNLOAD;
    return this.restClientService.postResource(url, String.class);
  }
}
