/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Solution - SolutionRestTestsAndDocumentation.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.test.doc;

import org.junit.jupiter.api.extension.ExtendWith;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.context.SpringBootTest.WebEnvironment;
import org.springframework.context.annotation.Import;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.context.junit.jupiter.SpringExtension;

import ch.unige.solidify.IndexUrlPath;
import ch.unige.solidify.OAIUrlPath;
import ch.unige.solidify.test.doc.RestDocsDocumentation;

import ch.dlcm.rest.UrlPath;
import ch.dlcm.test.resource.AipDoc;
import ch.dlcm.test.resource.AipDownloadDoc;
import ch.dlcm.test.resource.ApplicationRoleDoc;
import ch.dlcm.test.resource.ArchiveACLDoc;
import ch.dlcm.test.resource.ArchiveStatisticsDoc;
import ch.dlcm.test.resource.ArchiveTypeDoc;
import ch.dlcm.test.resource.ArchiveUserRatingDoc;
import ch.dlcm.test.resource.ContributorDoc;
import ch.dlcm.test.resource.DepositDoc;
import ch.dlcm.test.resource.DipDoc;
import ch.dlcm.test.resource.DisseminationPolicyDoc;
import ch.dlcm.test.resource.FundingAgencyDoc;
import ch.dlcm.test.resource.GlobalBannerDoc;
import ch.dlcm.test.resource.IndexFieldAliasDoc;
import ch.dlcm.test.resource.InstitutionDoc;
import ch.dlcm.test.resource.LanguageDoc;
import ch.dlcm.test.resource.LicenseDoc;
import ch.dlcm.test.resource.MetadataDoc;
import ch.dlcm.test.resource.MetadataTypeDoc;
import ch.dlcm.test.resource.ModuleDoc;
import ch.dlcm.test.resource.NotificationDoc;
import ch.dlcm.test.resource.OaiMetadataPrefixDoc;
import ch.dlcm.test.resource.OaiSetDoc;
import ch.dlcm.test.resource.OrderDoc;
import ch.dlcm.test.resource.OrgUnitDoc;
import ch.dlcm.test.resource.OrganizationalUnitDoc;
import ch.dlcm.test.resource.PersonDoc;
import ch.dlcm.test.resource.PreservationJobDoc;
import ch.dlcm.test.resource.PreservationPolicyDoc;
import ch.dlcm.test.resource.RatingTypeDoc;
import ch.dlcm.test.resource.ResearchDomainDoc;
import ch.dlcm.test.resource.RoleDoc;
import ch.dlcm.test.resource.ScheduledTaskDoc;
import ch.dlcm.test.resource.SipDoc;
import ch.dlcm.test.resource.SubmissionAgreementDoc;
import ch.dlcm.test.resource.SubmissionPolicyDoc;
import ch.dlcm.test.resource.SystemPropertyDoc;
import ch.dlcm.test.resource.UserDoc;

@ActiveProfiles({ "ff-notool", "vir-notool", "pre-zip", "ing-zip", "arc-file", "index-inmemory", "test", "sec-noauth", "client-standalone" })
@ExtendWith(SpringExtension.class)
@Import(SolutionRestTestsAndDocumentationConfig.class)
@SpringBootTest(webEnvironment = WebEnvironment.DEFINED_PORT, properties = { "spring.cloud.bootstrap.enabled=true" })
public class SolutionRestTestsAndDocumentation extends RestDocsDocumentation {

  @Override
  protected void loadResources() {
    this.addResources(new AipDoc());
    this.addResources(new AipDownloadDoc());
    this.addResources(new ArchiveACLDoc());
    this.addResources(new ArchiveStatisticsDoc());
    this.addResources(new ArchiveUserRatingDoc());
    this.addResources(new DepositDoc());
    this.addResources(new DipDoc());
    this.addResources(new DisseminationPolicyDoc());
    this.addResources(new FundingAgencyDoc());
    this.addResources(new IndexFieldAliasDoc());
    this.addResources(new InstitutionDoc());
    this.addResources(new LanguageDoc());
    this.addResources(new LicenseDoc());
    this.addResources(new NotificationDoc());
    this.addResources(new OrderDoc());
    this.addResources(new OrganizationalUnitDoc());
    this.addResources(new OrgUnitDoc());
    this.addResources(new OaiMetadataPrefixDoc());
    this.addResources(new OaiSetDoc());
    this.addResources(new PersonDoc());
    this.addResources(new PreservationJobDoc());
    this.addResources(new PreservationPolicyDoc());
    this.addResources(new RatingTypeDoc());
    this.addResources(new ResearchDomainDoc());
    this.addResources(new ApplicationRoleDoc());
    this.addResources(new RoleDoc());
    this.addResources(new SipDoc());
    this.addResources(new SubmissionAgreementDoc());
    this.addResources(new SubmissionPolicyDoc());
    this.addResources(new SystemPropertyDoc());
    this.addResources(new UserDoc());
    this.addResources(new ResearchDomainDoc());
    this.addResources(new ArchiveTypeDoc());
    this.addResources(new MetadataTypeDoc());
    this.addResources(new ContributorDoc());
    this.addResources(new MetadataDoc());
    this.addResources(new ModuleDoc());
    this.addResources(new ScheduledTaskDoc());
    this.addResources(new GlobalBannerDoc());
  }

  @Override
  protected void setUrlIgnoreList() {
    // Admin
    this.urlIgnoreList.add(this.getContextPath() + UrlPath.ADMIN_AUTHORIZED_ORG_UNIT);
    this.urlIgnoreList.add(this.getContextPath() + UrlPath.ADMIN_AUTHORIZED_INSTITUTION);
    // Access
    this.urlIgnoreList.add(this.getContextPath() + OAIUrlPath.OAI_PROVIDER);
    this.urlIgnoreList.add(this.getContextPath() + UrlPath.ACCESS_AIP);
    this.urlIgnoreList.add(this.getContextPath() + UrlPath.ACCESS_ALL_METADATA);
    this.urlIgnoreList.add(this.getContextPath() + UrlPath.ACCESS_PRIVATE_METADATA);
    // Archival Storage
    this.urlIgnoreList.add(this.getContextPath() + UrlPath.ARCHIVAL_STORAGE_STORED_AIP);
    // Preservation Planning
    this.urlIgnoreList.add(this.getContextPath() + UrlPath.PRES_PLANNING_MONITOR);
    this.urlIgnoreList.add(this.getContextPath() + UrlPath.PRES_PLANNING_AIP);
    // Data Management
    this.urlIgnoreList.add(this.getContextPath() + UrlPath.DATA_MGMT_DI);
    this.urlIgnoreList.add(this.getContextPath() + UrlPath.DATA_MGMT_PUBLIC_DI);
    this.urlIgnoreList.add(this.getContextPath() + UrlPath.DATA_MGMT_PRIVATE_DI);
    this.urlIgnoreList.add(this.getContextPath() + UrlPath.DATA_MGMT_ITEM);
    this.urlIgnoreList.add(this.getContextPath() + UrlPath.DATA_MGMT_ARCHIVE_THUMBNAIL);
    // Index
    this.urlIgnoreList.add(this.getContextPath() + IndexUrlPath.INDEX_SETTING);
    this.urlIgnoreList.add(this.getContextPath() + IndexUrlPath.INDEX_INDEX);
  }

  @Override
  protected void setUrlReadOnlyList() {
    // Admin
    this.urlReadOnlyList.add(this.getContextPath() + UrlPath.ADMIN_SYSTEM_PROPERTY);
    this.urlReadOnlyList.add(this.getContextPath() + UrlPath.ADMIN_ARCHIVE_STATISTICS);
    // Access
    this.urlReadOnlyList.add(this.getContextPath() + UrlPath.ACCESS_ORG_UNIT);
    this.urlReadOnlyList.add(this.getContextPath() + UrlPath.ACCESS_PUBLIC_METADATA);
    // Preingest
    this.urlReadOnlyList.add(this.getContextPath() + UrlPath.PREINGEST_CONTRIBUTOR);
    // Preservation Planning;
    this.urlReadOnlyList.add(this.getContextPath() + UrlPath.PRES_PLANNING_MODULE);
  }

}
