/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Solution - AipDownloadDoc.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.test.resource;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.springframework.restdocs.payload.FieldDescriptor;

import ch.unige.solidify.SolidifyConstants;
import ch.unige.solidify.rest.ActionName;
import ch.unige.solidify.test.helper.AbstractResourceDoc;
import ch.unige.solidify.test.helper.FieldDescriptorBuilder;
import ch.unige.solidify.test.model.TypeField;

import ch.dlcm.model.oais.ArchiveContainer;
import ch.dlcm.rest.DLCMActionName;
import ch.dlcm.rest.ResourceName;
import ch.dlcm.rest.UrlPath;

public class AipDownloadDoc extends AbstractResourceDoc {

  @Override
  public List<String> getListLinks() {
    return Arrays.asList(SolidifyConstants.SCHEMA, ActionName.VALUES, DLCMActionName.ARCHIVAL_COLLECTIONS, DLCMActionName.ARCHIVAL_UNITS);
  }

  @Override
  public String getName() {
    return UrlPath.ACCESS_AIP;
  }

  @Override
  public List<String> getResourceLinks() {
    return Arrays.asList(ResourceName.DATAFILE, ActionName.UPLOAD, ActionName.RESUME, ActionName.HISTORY);
  }

  @Override
  protected List<FieldDescriptor> getDocumentationFields() {
    // @formatter:off
    final List<FieldDescriptor> documentationFields = new ArrayList<>(Arrays.asList(
      new FieldDescriptorBuilder(this)
        .name("aipContainer")
        .description("The container type of the AIP")
        .mandatory(true)
        .enumValues(this.enumToList(ArchiveContainer.values()))
        .testValue(ArchiveContainer.BAG_IT.toString())
        .build(),
      new FieldDescriptorBuilder(this)
        .name("archivalUnit")
        .description("if the AIP is an unit (AIU) or a collection (AIC)")
        .type(TypeField.BOOLEAN)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("sipIds")
        .description("The source SIP IDs of the AIP")
        .type(TypeField.ARRAY)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("archiveId")
        .description("The storage URL of the AIP")
        .type(TypeField.STRING)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("allDataFileReady")
        .description("If all data file are ready")
        .build(),
      new FieldDescriptorBuilder(this)
        .name("dataFileNumber")
        .description("Number of AIP data files")
        .type(TypeField.INT32)
        .build()));
    // @formatter:on
    documentationFields.addAll(new InformationPackageDoc().getDocumentationFields());
    return documentationFields;
  }

}
