/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Solution - LicenseDoc.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.test.resource;

import java.util.Arrays;
import java.util.List;

import org.springframework.restdocs.payload.FieldDescriptor;
import org.springframework.restdocs.payload.JsonFieldType;

import ch.unige.solidify.rest.ActionName;
import ch.unige.solidify.test.helper.AbstractResourceDoc;
import ch.unige.solidify.test.helper.FieldDescriptorBuilder;
import ch.unige.solidify.test.model.TypeField;

import ch.dlcm.model.IdentifierType;
import ch.dlcm.model.settings.License.ConformanceStatus;
import ch.dlcm.model.settings.License.LicenseStatus;
import ch.dlcm.rest.UrlPath;

public class LicenseDoc extends AbstractResourceDoc {

  @Override
  public List<String> getListLinks() {
    return Arrays.asList(ActionName.IMPORT);
  }

  @Override
  public String getName() {
    return UrlPath.ADMIN_LICENSE;
  }

  @Override
  public List<String> getResourceLinks() {
    return Arrays.asList(IdentifierType.SPDX_ID.toString().toLowerCase());
  }

  @Override
  protected List<FieldDescriptor> getDocumentationFields() {
    // @formatter:off
    return Arrays.asList(
      new FieldDescriptorBuilder(this)
        .name("openLicenseId")
        .description("The open license identifier")
        .mandatory(true)
        .customConstraints("SPDX short identifier")
        .type(TypeField.STRING)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("title")
        .description("The title of the license")
        .mandatory(true)
        .type(TypeField.STRING)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("description")
        .description("The description of the license")
        .type(TypeField.STRING)
        .customConstraints("<openLicense> (<title>)")
        .build(),
      new FieldDescriptorBuilder(this)
        .name("domainContent")
        .description("The content of the domain")
        .mandatory(true)
        .type(TypeField.BOOLEAN)
        .build().type(JsonFieldType.BOOLEAN),
      new FieldDescriptorBuilder(this)
        .name("domainData")
        .description("The data domain")
        .mandatory(true)
        .type(TypeField.BOOLEAN)
        .build().type(JsonFieldType.BOOLEAN),
      new FieldDescriptorBuilder(this)
        .name("domainSoftware")
        .description("The domain software")
        .mandatory(true)
        .type(TypeField.BOOLEAN)
        .build().type(JsonFieldType.BOOLEAN),
      new FieldDescriptorBuilder(this)
        .name("family")
        .description("The family license")
        .type(TypeField.STRING)
        .build().type(JsonFieldType.STRING),
      new FieldDescriptorBuilder(this)
        .name("maintainer")
        .description("The maintainer of the license")
        .type(TypeField.STRING)
        .build().type(JsonFieldType.STRING),
      new FieldDescriptorBuilder(this)
        .name("isGeneric")
        .description("Give if it's a generic license")
        .mandatory(true)
        .type(TypeField.BOOLEAN)
        .build().type(JsonFieldType.BOOLEAN),
      new FieldDescriptorBuilder(this)
        .name("url")
        .description("The URL source of the license")
        .type(TypeField.URL)
        .customConstraints(CONSTRAINT_VALID_URL)
        .testValue(TEST_URL_VALUE)
        .build().type(JsonFieldType.STRING),
      new FieldDescriptorBuilder(this)
        .name("status")
        .description("The current status of the license")
        .type(TypeField.URL)
        .enumValues(this.enumToList(LicenseStatus.values(),true))
        .build().type(JsonFieldType.STRING),
      new FieldDescriptorBuilder(this)
         .name("logo")
         .description("Logo of the license")
         .type(TypeField.UNDEFINED)
         .build(),
      new FieldDescriptorBuilder(this)
        .name("odConformance")
        .description("The OD conformance status")
        .type(TypeField.URL)
        .enumValues(this.enumToList(Arrays.stream(ConformanceStatus.values()).map(ConformanceStatus::getValue).toArray(String[]::new), true))
        .build().type(JsonFieldType.STRING),
      new FieldDescriptorBuilder(this)
        .name("osdConformance")
        .description("The OSD conformance status")
        .type(TypeField.URL)
        .enumValues(this.enumToList(Arrays.stream(ConformanceStatus.values()).map(c -> c.getValue()).toArray(String[]::new), true))
        .build().type(JsonFieldType.STRING));
    // @formatter:on
  }

}
