/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Solution - MetadataDoc.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.test.resource;

import static org.springframework.restdocs.payload.PayloadDocumentation.subsectionWithPath;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.springframework.restdocs.payload.FieldDescriptor;
import org.springframework.restdocs.snippet.Attributes;

import ch.unige.solidify.rest.ActionName;
import ch.unige.solidify.test.helper.AbstractResourceDoc;
import ch.unige.solidify.test.helper.FieldDescriptorBuilder;
import ch.unige.solidify.test.helper.RestDocsTemplates;
import ch.unige.solidify.test.model.TypeField;

import ch.dlcm.model.Access;
import ch.dlcm.rest.DLCMActionName;
import ch.dlcm.rest.ResourceName;
import ch.dlcm.rest.UrlPath;

public class MetadataDoc extends AbstractResourceDoc {

  @Override
  public List<String> getListLinks() {
    return Arrays.asList(ActionName.VALUES, ActionName.SEARCH, DLCMActionName.SEARCH_DOI);
  }

  @Override
  public String getName() {
    return UrlPath.ACCESS_PUBLIC_METADATA;
  }

  @Override
  public List<String> getResourceLinks() {
    return Arrays.asList(DLCMActionName.PREPARE_DOWNLOAD, DLCMActionName.DOWNLOAD_STATUS, ActionName.DOWNLOAD, DLCMActionName.THUMBNAIL,
            DLCMActionName.STATISTICS, ResourceName.PACKAGES, ResourceName.DATAFILE, ResourceName.AIP, DLCMActionName.LIST_FOLDERS,
            DLCMActionName.RATING, DLCMActionName.BIBLIOGRAPHIES, DLCMActionName.CITATIONS);
  }

  @Override
  protected List<FieldDescriptor> getDocumentationFields() {
    // @formatter:off
    return new ArrayList<>(Arrays.asList(
      new FieldDescriptorBuilder(this)
        .name("index")
        .description("The index name of indexed metadata")
        .type(TypeField.STRING)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("currentAccess")
        .description("The current access level of indexed metadata")
        .enumValues(this.enumToList(Access.values()))
        .build(),
      subsectionWithPath("metadata")
        .description("The object of indexed metadata")
        .attributes(new Attributes.Attribute(READ_ONLY_VALUE, String.valueOf(true)))));
    // @formatter:on
  }

  @Override
  public List<String> getCommonResourceFields() {
    return List.of(RestDocsTemplates.RESOURCE_ID_FIELD, RestDocsTemplates.LINKS_FIELD);
  }
}
