/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Solution - NotificationDoc.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.test.resource;

import static org.springframework.restdocs.payload.PayloadDocumentation.subsectionWithPath;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

import org.springframework.restdocs.payload.FieldDescriptor;
import org.springframework.restdocs.snippet.Attributes;

import ch.unige.solidify.rest.ActionName;
import ch.unige.solidify.test.helper.AbstractResourceDoc;
import ch.unige.solidify.test.helper.FieldDescriptorBuilder;
import ch.unige.solidify.test.model.TypeField;

import ch.dlcm.model.notification.NotificationStatus;
import ch.dlcm.model.notification.NotificationType;
import ch.dlcm.rest.UrlPath;

public class NotificationDoc extends AbstractResourceDoc {

  @Override
  public String getName() {
    return UrlPath.ADMIN_NOTIFICATION;
  }

  @Override
  protected List<FieldDescriptor> getDocumentationFields() {
    // @formatter:off
    return new ArrayList<>(Arrays.asList(
      subsectionWithPath("notificationType").description("The type of the notification")
        .attributes(new Attributes.Attribute(READ_ONLY_VALUE, String.valueOf(true)),
                    new Attributes.Attribute(MANDATORY_VALUE, String.valueOf(true))),
      new FieldDescriptorBuilder(this)
        .name("notificationType.resId")
        .description("The type of the notification")
        .mandatory(true)
        .type(TypeField.STRING)
        .enumValues(NotificationType.getAllNotificationTypes().stream().map(NotificationType::toString).collect(Collectors.toList()))
        .customConstraints(CONSTRAINT_VALID_JSON)
        .testValue("ACCESS_DATASET_REQUEST")
        .readOnly(true)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("notificationStatus")
        .description("The status of the notification")
        .mandatory(true)
        .type(TypeField.STRING)
        .enumValues(this.enumToList(NotificationStatus.values()))
        .customConstraints(null)
        .testValue(null)
        .readOnly(true)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("statusMessage")
        .description("The message related to the status")
        .type(TypeField.STRING)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("message")
        .description("The message sent by the notification")
        .mandatory(true)
        .type(TypeField.STRING)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("objectId")
        .description("The target object ID of the notification")
        .mandatory(true)
        .type(TypeField.STRING)
        .build(),
      subsectionWithPath("emitter")
        .description("The emitter of the notification (User)")
        .attributes(new Attributes.Attribute(READ_ONLY_VALUE, String.valueOf(true))),
      new FieldDescriptorBuilder(this)
        .name("recipient.resId")
        .description("The recipient of the notification (Person id)")
        .mandatory(false)
        .type(TypeField.STRING)
        .testValue("dlcm")
        .readOnly(true)
        .build(),
      subsectionWithPath("recipient")
         .description("The recipient of the notification (Person)")
         .attributes(new Attributes.Attribute(READ_ONLY_VALUE, String.valueOf(true)),
                 new Attributes.Attribute(MANDATORY_VALUE, String.valueOf(true))),
      new FieldDescriptorBuilder(this)
         .name("sentTime")
         .description("Time when the notification was sent")
         .type(TypeField.UNDEFINED)
         .build(),
      new FieldDescriptorBuilder(this)
        .name("notifiedOrgUnit.resId")
        .description("The ID of the notified organizational unit")
        .mandatory(false)
        .type(TypeField.STRING)
        .enumValues(null)
        .customConstraints("")
        .testValue(TEST_STRING_VALUE)
        .readOnly(true)
        .build(),
      subsectionWithPath("notifiedOrgUnit")
        .description("The notified organizational unit")
        .attributes(new Attributes.Attribute(READ_ONLY_VALUE, String.valueOf(true)),
                    new Attributes.Attribute(MANDATORY_VALUE, String.valueOf(true)))));
    // @formatter:on
  }

  @Override
  public List<String> getResourceLinks() {
    return Arrays.asList(ActionName.HISTORY);
  }

  @Override
  public List<String> getListLinks() {
    return Collections.emptyList();
  }

}
