/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Solution - OrderDoc.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.test.resource;

import java.util.Arrays;
import java.util.List;

import org.springframework.restdocs.payload.FieldDescriptor;

import ch.unige.solidify.rest.ActionName;
import ch.unige.solidify.test.helper.AbstractResourceDoc;
import ch.unige.solidify.test.helper.FieldDescriptorBuilder;
import ch.unige.solidify.test.model.TypeField;

import ch.dlcm.DLCMMetadataVersion;
import ch.dlcm.model.access.Order;
import ch.dlcm.model.access.QueryType;
import ch.dlcm.rest.ResourceName;
import ch.dlcm.rest.UrlPath;

public class OrderDoc extends AbstractResourceDoc {

  @Override
  public List<String> getListLinks() {
    return Arrays.asList(ActionName.VALUES);
  }

  @Override
  public String getName() {
    return UrlPath.ACCESS_ORDER;
  }

  @Override
  public List<String> getResourceLinks() {
    return Arrays.asList(ResourceName.AIP, ActionName.VIEW, ActionName.SAVE, ActionName.HISTORY);
  }

  @Override
  protected List<FieldDescriptor> getDocumentationFields() {
    // @formatter:off
    return Arrays.asList(
      new FieldDescriptorBuilder(this)
        .name("name")
        .description("The name of the order query")
        .mandatory(true)
        .type(TypeField.STRING)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("publicOrder")
        .description("True if the order is public, false otherwise")
        .type(TypeField.BOOLEAN)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("queryType")
        .description("The type of the order query")
        .mandatory(true)
        .type(TypeField.STRING)
        .enumValues(this.enumToList(QueryType.values()))
        .customConstraints(null)
        .testValue(null)
        .readOnly(true)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("query")
        .description("The query of the order query")
        .mandatory(true)
        .type(TypeField.STRING)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("status")
        .description("The status of the order query")
        .enumValues(this.enumToList(Order.OrderStatus.values()))
        .build(),
      new FieldDescriptorBuilder(this)
        .name("statusMessage")
        .description("The message related to the status")
        .type(TypeField.STRING)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("aipNumber")
        .description("The AIP number of the order query")
        .type(TypeField.INT32)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("dipNumber")
        .description("The DIP number of the order query")
        .type(TypeField.INT32)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("metadataVersion")
        .description("The metadata version of the deposit")
        .enumValues(this.enumToList(DLCMMetadataVersion.values()))
        .testValue(DLCMMetadataVersion.getDefaultVersion().getVersion())
        .build());
    // @formatter:on
  }

}
