/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Solution - OrgUnitDoc.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.test.resource;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import org.springframework.restdocs.payload.FieldDescriptor;
import org.springframework.restdocs.payload.JsonFieldType;

import ch.unige.solidify.test.helper.AbstractResourceDoc;
import ch.unige.solidify.test.helper.FieldDescriptorBuilder;
import ch.unige.solidify.test.model.TypeField;

import ch.dlcm.rest.DLCMActionName;
import ch.dlcm.rest.ResourceName;
import ch.dlcm.rest.UrlPath;

public class OrgUnitDoc extends AbstractResourceDoc {

  @Override
  public List<String> getListLinks() {
    return Collections.emptyList();
  }

  @Override
  public String getModelName() {
    return "access-organizational-unit";
  }

  @Override
  public String getName() {
    return UrlPath.ACCESS_ORG_UNIT;
  }

  @Override
  public List<String> getResourceLinks() {
    return Arrays.asList(ResourceName.ARCHIVE, DLCMActionName.DOWNLOAD_LOGO);
  }

  @Override
  protected List<FieldDescriptor> getDocumentationFields() {
    return new ArrayList<>(Arrays.asList(
    // @formatter:off
      new FieldDescriptorBuilder(this)
        .name("name")
        .description("The name of the organizational unit")
        .mandatory(true)
        .type(TypeField.STRING)
        .customConstraints("The name of the organizational unit")
        .build(),
      new FieldDescriptorBuilder(this)
        .name("description")
        .description("The description of the institution")
        .type(TypeField.STRING)
        .build().type(JsonFieldType.STRING),
      new FieldDescriptorBuilder(this)
        .name("openingDate")
        .description("The opening date of the organizational unit")
        .type(TypeField.DATE)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("closingDate")
        .description("The closing date of the organizational unit")
        .type(TypeField.DATE)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("url")
        .description("The URL of the organizational unit")
        .type(TypeField.URL)
        .customConstraints(CONSTRAINT_VALID_URL)
        .testValue(TEST_URL_VALUE)
        .build().type(JsonFieldType.STRING),
      new FieldDescriptorBuilder(this)
        .name("logo")
        .description("The logo of the organizational unit")
        .type(TypeField.UNDEFINED)
        .build()));
    // @formatter:on
  }
}
