/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Solution - PreservationJobDoc.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.test.resource;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.springframework.restdocs.payload.FieldDescriptor;
import org.springframework.restdocs.payload.JsonFieldType;

import ch.unige.solidify.rest.ActionName;
import ch.unige.solidify.test.helper.AbstractResourceDoc;
import ch.unige.solidify.test.helper.FieldDescriptorBuilder;
import ch.unige.solidify.test.model.TypeField;

import ch.dlcm.model.preservation.JobRecurrence;
import ch.dlcm.model.preservation.JobType;
import ch.dlcm.model.preservation.PreservationJob.JobStatus;
import ch.dlcm.rest.ResourceName;
import ch.dlcm.rest.UrlPath;

public class PreservationJobDoc extends AbstractResourceDoc {

  @Override
  public List<String> getListLinks() {
    return Arrays.asList(ActionName.VALUES);
  }

  @Override
  public String getName() {
    return UrlPath.PRES_PLANNING_PRES_JOB;
  }

  @Override
  public List<String> getResourceLinks() {
    return Arrays.asList(ActionName.START, ResourceName.PRES_JOB_EXECUTION);
  }

  @Override
  protected List<FieldDescriptor> getDocumentationFields() {
    // @formatter:off
    final List<FieldDescriptor> documentationFields = new ArrayList<>( Arrays.asList(
      new FieldDescriptorBuilder(this)
        .name("name")
        .description("The name of the preservation job")
        .mandatory(true)
        .type(TypeField.STRING)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("jobType")
        .description("The job type of the preservation job")
        .mandatory(true)
        .enumValues(this.enumToList(JobType.values()))
        .testValue(JobType.REPLICATION.toString())
        .build(),
      new FieldDescriptorBuilder(this)
        .name("jobRecurrence")
        .description("Job recurrence")
        .mandatory(false)
        .enumValues(this.enumToList(JobRecurrence.values()))
        .testValue(JobRecurrence.YEARLY.toString())
        .build(),
      new FieldDescriptorBuilder(this)
        .name("enable")
        .description("If the job is enable")
        .mandatory(false)
        .type(TypeField.BOOLEAN)
        .build().type(JsonFieldType.BOOLEAN),
      new FieldDescriptorBuilder(this)
        .name("executionNumber")
        .description("Number of job executions")
        .mandatory(false)
        .type(TypeField.INT32)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("lastExecutionStatus")
        .description("The status of the last job execution")
        .mandatory(false)
        .enumValues(this.enumToList(JobStatus.values()))
        .testValue(JobStatus.COMPLETED.toString())
        .build()));
    // @formatter:on
    documentationFields.addAll(new JobSchedulingDoc().getDocumentationFields());
    return documentationFields;
  }
}
