/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Solution - SubmissionPolicyDoc.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.test.resource;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import org.springframework.restdocs.payload.FieldDescriptor;

import ch.unige.solidify.test.helper.AbstractResourceDoc;
import ch.unige.solidify.test.helper.FieldDescriptorBuilder;
import ch.unige.solidify.test.model.TypeField;

import ch.dlcm.model.schedule.TaskType;
import ch.dlcm.rest.UrlPath;

public class ScheduledTaskDoc extends AbstractResourceDoc {

  @Override
  public List<String> getListLinks() {
    return Collections.emptyList();
  }

  @Override
  public String getName() {
    return UrlPath.ADMIN_SCHEDULED_TASKS;
  }

  @Override
  public List<String> getResourceLinks() {
    return Collections.emptyList();
  }

  @Override
  protected List<FieldDescriptor> getDocumentationFields() {
    return Arrays.asList(
    // @formatter:off
      new FieldDescriptorBuilder(this)
        .name("name")
        .description("The name of the scheduled task")
        .mandatory(true)
        .type(TypeField.STRING)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("taskType")
        .description("The type of the scheduled task")
        .mandatory(true)
        .enumValues(this.enumToList(TaskType.values()))
        .testValue(TaskType.NOTIFY_DEPOSITS_TO_VALIDATE.toString())
        .type(TypeField.STRING)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("cronExpression")
        .description("The CRON expression of the scheduled task")
        .mandatory(true)
        .testValue("* * * * * *")
        .customConstraints("Seconds Minutes Hours Day-of-month Month Year") 
        .type(TypeField.STRING)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("lastExecutionStart")
        .description("The last execution start of the scheduled task")
        .type(TypeField.DATE_TIME)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("lastExecutionEnd")
        .description("The last execution end of the scheduled task")
        .type(TypeField.DATE_TIME)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("nextExecution")
        .description("The next execution date of the scheduled task")
        .type(TypeField.DATE_TIME)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("enabled")
        .description("If the task is enable")
        .type(TypeField.BOOLEAN)
        .build());
    // @formatter:on
  }
}
