/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Solution - SipDoc.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.dlcm.test.resource;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.springframework.restdocs.payload.FieldDescriptor;
import org.springframework.restdocs.payload.JsonFieldType;

import ch.unige.solidify.SolidifyConstants;
import ch.unige.solidify.rest.ActionName;
import ch.unige.solidify.test.helper.AbstractResourceDoc;
import ch.unige.solidify.test.helper.FieldDescriptorBuilder;
import ch.unige.solidify.test.model.TypeField;

import ch.dlcm.DLCMConstants;
import ch.dlcm.rest.ResourceName;
import ch.dlcm.rest.UrlPath;

public class SipDoc extends AbstractResourceDoc {

  @Override
  public List<String> getListLinks() {
    return Arrays.asList(SolidifyConstants.SCHEMA, ResourceName.PROFILE, ActionName.VALUES);
  }

  @Override
  public String getName() {
    return UrlPath.INGEST_SIP;
  }

  @Override
  public List<String> getResourceLinks() {
    return Arrays.asList(ResourceName.DATAFILE, ResourceName.AIP, ActionName.UPLOAD, ActionName.RESUME, ActionName.HISTORY);
  }

  @Override
  protected List<FieldDescriptor> getDocumentationFields() {
    // @formatter:off
    final List<FieldDescriptor> documentationFields = new ArrayList<>(Arrays.asList(
      new FieldDescriptorBuilder(this)
        .name("depositId")
        .description("The source deposit ID of the SIP")
        .type(TypeField.STRING)
        .build().type(JsonFieldType.STRING),
      new FieldDescriptorBuilder(this)
        .name(DLCMConstants.DB_AIP_ID)
        .description("The generated AIP DLCM ID of the SIP")
        .type(TypeField.STRING)
        .build().type(JsonFieldType.STRING),
      new FieldDescriptorBuilder(this)
        .name("retention")
        .description("The retention duration in days of the AIP")
        .mandatory(true)
        .type(TypeField.INT32)
        .customConstraints(CONSTRAINT_DATE_INTEGER)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("submissionPolicy")
        .description("The submission policy object of the SIP")
        .type(TypeField.UNDEFINED)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("submissionPolicyId")
        .description("The submission policy ID of the SIP")
        .type(TypeField.STRING)
        .customConstraints(CONSTRAINT_VALID_ID)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("organizationalUnit")
        .description("The organizational unit object of the SIP")
        .type(TypeField.UNDEFINED)
        .build(),
      new FieldDescriptorBuilder(this)
        .name(DLCMConstants.ORG_UNIT_ID_FIELD)
        .description("The organizational unit ID of the SIP")
        .mandatory(true)
        .type(TypeField.STRING)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("smartRetention")
        .description("The retention duration in human-readable format of the AIP")
        .type(TypeField.STRING)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("dispositionApproval")
        .description("Mandatory step to approve the AIP disposition")
        .type(TypeField.BOOLEAN)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("ready")
        .description("If the SIP package is ready")
        .type(TypeField.BOOLEAN)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("collectionSize")
        .description("Size of AIP collection")
        .type(TypeField.INT32)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("dataFileNumber")
        .description("Number of SIP data files")
        .type(TypeField.INT32)
        .build()));
    // @formatter:on
    documentationFields.addAll(new InformationPackageDoc().getDocumentationFields());
    return documentationFields;
  }

}
